@@ -12,7 +12,7 @@ from .load_backend import  get_session , get_trident_dir , epsilon , set_epsilon
 
 from .load_backend import tile_rgb_images,loss_metric_curve,steps_histogram
 from .load_backend import read_image , read_mask , save_image , save_mask , image2array , array2image , mask2array , array2mask , list_pictures , normalize , unnormalize , random_crop , resize , add_noise , image_backend_adaptive , random_channel_shift , random_cutout
-from ..optims.trainers import ModelBase,OptimizerMixin,TrainingPlan
+from ..optims.trainers import ModelBase,OptimizerBase,TrainingPlan
 
 
 from  ..optims.trainers import *
