@@ -20,11 +20,17 @@ from pydoc import locate
 import datetime
 import time
 import types
-
+import traceback
+import json
 __all__ = ['get_session','get_trident_dir','epsilon','set_epsilon','floatx','set_floatx','if_else','camel2snake','snake2camel','to_onehot','to_list','addindent','format_time', 'get_time_suffix', 'get_function', 'get_class', 'get_terminal_size', 'gcd', 'get_divisors', 'isprime', 'next_prime', 'prev_prime', 'nearest_prime','PrintException','unpack_singleton','enforce_singleton','OrderedDict','get_python_function_arguments','map_function_arguments','ClassfierType','PaddingMode', 'DataRole',
 
            'ExpectDataType', 'GetImageMode', 'make_dir_if_need', 'ShortcutMode', 'DataSpec','get_argument_maps','get_args_spec']
 
+
+
+
+
+
 if 'TRIDENT_HOME' in os.environ:
     _trident_dir = os.environ.get('TRIDENT_HOME')
     if not os.path.exists(_trident_dir):
@@ -41,9 +47,50 @@ else:
     _trident_dir = os.path.join(_trident_base_dir, '.trident')
 
 
+# Attempt to read Trident config file.
+
+
+
+
 _SESSION = threading.local()
 
 _SESSION.trident_dir=_trident_dir
+_SESSION.backend='pytorch'
+_SESSION.image_backend ='pillow'
+_SESSION.epoch_equivalent=200
+_SESSION.floatx='float32'
+_SESSION.epsilon= 1e-8
+_SESSION.numpy_print_format='{0:.4e}'
+
+
+
+
+_config_path = os.path.expanduser(os.path.join(_trident_dir, 'trident.json'))
+if os.path.exists(_config_path):
+    try:
+        with open(_config_path) as f:
+            _config = json.load(f)
+    except ValueError:
+        _config = {}
+    for k,v in _config.items():
+        try:
+            if k=='floatx':
+                assert v in {'float16', 'float32', 'float64'}
+            _SESSION.__setattr__(k,v)
+        except :
+            exc_type, exc_obj, tb = sys.exc_info()
+            traceback.print_exception(exc_type, exc_obj, tb, limit=2, file=sys.stdout)
+
+
+
+
+
+
+
+
+
+
+
 
 
 from enum import Enum, unique
@@ -60,11 +107,7 @@ class IntevalUnit(Enum):
     epoch = 'epoch'
     once = 'once'
 
-
-
-np.set_printoptions(formatter={'float_kind':lambda x: '{0:.4e}'.format(x)})
-
-
+np.set_printoptions(formatter={'float_kind':lambda x: _SESSION.numpy_print_format.format(x)})
 
 
 def get_plateform():
@@ -82,24 +125,21 @@ def get_session():
 def get_trident_dir():
     return _trident_dir
 
-_SESSION.floatx='float32'
-_SESSION.epsilon= 1e-8
-# the type of float to use throughout the session.
-
-_SESSION.backend='pytorch'
-_SESSION.image_data_format='channels_first'
-_SESSION.image_channel_order='bgr'
-
 
 
 def PrintException():
     exc_type, exc_obj, tb = sys.exc_info()
+
     f = tb.tb_frame
     lineno = tb.tb_lineno
     filename = f.f_code.co_filename
     linecache.checkcache(filename)
     line = linecache.getline(filename, lineno, f.f_globals)
     print('EXCEPTION IN ({}, LINE {} "{}"): {}'.format(filename, lineno, line.strip(), exc_obj))
+    traceback.print_tb(tb, limit=1, file=sys.stdout)
+    traceback.print_exception(exc_type, exc_obj, tb, limit=3, file=sys.stdout)
+
+
 
 def epsilon():
     """Returns the value of the fuzz factor used in numeric expressions.
