@@ -1,60 +1,45 @@
 from __future__ import absolute_import
 from __future__ import print_function
-import json
+
 from sys import stderr
 import os
 from .common import *
 
 _session=get_session()
 _trident_dir=get_trident_dir()
+_config_path = os.path.expanduser(os.path.join(_trident_dir, 'trident.json'))
 
 
-
-
-
-# Default backend: Pytorch.
-_BACKEND = 'pytorch'
-_IMAGE_BACKEND='pillow'
-_session.backend = _BACKEND
-_session.image_backend = _IMAGE_BACKEND
-
 def write_config(_config_path):
-    _config = {
-        'floatx': _session.floatx,
-        'epsilon': _session.epsilon,
-        'backend': _session.backend ,
-        'image_backend':_session.image_backend
-    }
+    # _config = {
+    #     'floatx': _session.floatx,
+    #     'epsilon': _session.epsilon,
+    #     'backend': _session.backend ,
+    #     'image_backend':_session.image_backend
+    # }
     try:
         with open(_config_path, 'w') as f:
-            f.write(json.dumps(_config, indent=4))
+            f.write(json.dumps(_session, indent=4))
     except IOError:
         # Except permission denied.
         pass
 
-
-# Attempt to read Trident config file.
-_config_path = os.path.expanduser(os.path.join(_trident_dir, 'trident.json'))
-if os.path.exists(_config_path):
-    try:
-        with open(_config_path) as f:
-            _config = json.load(f)
-    except ValueError:
-        _config = {}
-    _floatx = _config.get('floatx', _session.floatx)
-    assert _floatx in {'float16', 'float32', 'float64'}
-    _epsilon = _config.get('epsilon', _session.epsilon)
-    assert isinstance(_epsilon, float)
-    _BACKEND = _config.get('backend', _session.backend )
-    _IMAGE_BACKEND = _config.get('image_backend', _session.image_backend)
+#
+# # Attempt to read Trident config file.
+# _config_path = os.path.expanduser(os.path.join(_trident_dir, 'trident.json'))
+# if os.path.exists(_config_path):
+#     try:
+#         with open(_config_path) as f:
+#             _config = json.load(f)
+#     except ValueError:
+#         _config = {}
+#     for k,v in _config.items():
+#         if k=='floatx':
+#             assert v in {'float16', 'float32', 'float64'}
+#         _session[k]=v
 
 
 
-    _session.floatx=_floatx
-    _session.epsilon=_epsilon
-    _session.backend =  _BACKEND
-    _session.image_backend =  _IMAGE_BACKEND
-
 # Save config file, if possible.
 if not os.path.exists(_trident_dir):
     try:
@@ -76,14 +61,13 @@ def get_image_backend():
 
 # Set backend based on TRIDENT_BACKEND flag, if applicable.
 if 'TRIDENT_BACKEND' in os.environ:
-    _BACKEND = os.environ['TRIDENT_BACKEND']
-    if _session.backend!=_BACKEND:
-        _session.backend = _BACKEND
+    if _session.backend!=os.environ['TRIDENT_BACKEND']:
+        _session.backend = os.environ['TRIDENT_BACKEND']
         write_config(_config_path)
 
 
 
-if _BACKEND == 'cntk':
+if _session.backend == 'cntk':
     stderr.write('Using CNTK backend\n')
     stderr.write('Image Data Format: channels_first.\n')
     stderr.write('Image Channel Order: rgb.\n')
@@ -96,7 +80,7 @@ if _BACKEND == 'cntk':
     #from ..optims.cntk_lr_schedulers import *
 
 
-elif _BACKEND == 'pytorch':
+elif _session.backend == 'pytorch':
     stderr.write('Using Pytorch backend.\n')
     stderr.write('Image Data Format: channels_first.\n')
     stderr.write('Image Channel Order: rgb.\n')
@@ -107,7 +91,7 @@ elif _BACKEND == 'pytorch':
     #layers=importlib.import_module('layers.pytorch_layers')
 
 
-elif _BACKEND == 'tensorflow':
+elif _session.backend == 'tensorflow':
     stderr.write('Using TensorFlow backend.\n')
     stderr.write('Image Data Format: channels_last.\n')
     stderr.write('Image Channel Order: rgb.\n')
@@ -116,16 +100,15 @@ elif _BACKEND == 'tensorflow':
     _session.image_channel_order = 'rgb'
 if 'TRIDENT_IMG_BACKEND' in os.environ:
     _image_backend = os.environ['TRIDENT_IMG_BACKEND']
-    if _image_backend:
-        _IMAGE_BACKEND = _image_backend
+    if _image_backend and _session.image_backend!=_image_backend:
         _session.image_backend = _image_backend
 
 
 
 
-if _IMAGE_BACKEND == 'opencv':
+if _session.image_backend == 'opencv':
     stderr.write('Using opencv image backend\n')
-elif _IMAGE_BACKEND == 'pillow':
+elif _session.image_backend== 'pillow':
     stderr.write('Using pillow image backend.\n')
 
 
