@@ -6,6 +6,8 @@ from  ..callbacks import CallbackBase
 from ..backend.common import *
 from ..backend.load_backend import get_backend
 
+_session = get_session()
+
 if get_backend()=='pytorch':
     from ..backend.pytorch_backend import to_numpy,to_tensor
 elif get_backend()=='tensorflow':
@@ -76,6 +78,7 @@ class ReduceLROnPlateau(AdjustLRCallbackBase):
             min_delta = kwargs.pop('epsilon')
             warnings.warn('`epsilon` argument is deprecated and '
                           'will be removed, use `min_delta` instead.')
+
         self.factor = factor
         self.min_lr = min_lr
         self.min_delta = min_delta
@@ -87,6 +90,7 @@ class ReduceLROnPlateau(AdjustLRCallbackBase):
         self.best = 0
         self.mode = mode
         self.monitor_op = None
+        self.unit_base=None
         self._reset()
 
     def _reset(self):
@@ -107,46 +111,58 @@ class ReduceLROnPlateau(AdjustLRCallbackBase):
         self.cooldown_counter = 0
         self.wait = 0
 
-    def on_train_begin(self, training_context):
-        self._reset()
-
     def on_epoch_end(self, training_context):
         training_context['current_lr']=training_context['optimizer'].lr
-        current =to_numpy(training_context['losses'].get(self.monitor,training_context['metrics'].get(self.monitor,training_context['losses']['total_losses']))).mean()
-        if current is None:
-            warnings.warn(
-                'Reduce LR on plateau conditioned on metric `%s` '
-                'which is not available. Available metrics are: %s' %
-                (self.monitor, ','.join(training_context['metrics'].keys_list)), RuntimeWarning
-            )
+        if self.unit_base == 'epoch':
+            history = training_context['losses'].get(self.monitor, training_context['metrics'].get(self.monitor,
+                                                                                                   training_context[
+                                                                                                       'losses'][
+                                                                                                       'total_losses']))
+            current = to_numpy(history[-min(5, len(history)):]).mean()
+            if current is None:
+                warnings.warn(
+                    'Reduce LR on plateau conditioned on metric `%s` '
+                    'which is not available. Available metrics are: %s' %
+                    (self.monitor, ','.join(training_context['metrics'].keys_list)), RuntimeWarning
+                )
 
-        else:
-            if self.in_cooldown():
-                self.cooldown_counter -= 1
-                self.wait = 0
+            else:
+                if self.in_cooldown():
+                    self.cooldown_counter -= 1
+                    self.wait = 0
 
-            if self.monitor_op(current, self.best):
-                self.best = current
-                self.wait = 0
-            elif not self.in_cooldown():
-                self.wait += 1
-                if self.wait >= self.patience:
-                    old_lr = float(training_context['optimizer'].lr)
-                    if old_lr > self.min_lr:
-                        new_lr = old_lr * self.factor
-                        new_lr = max(new_lr, self.min_lr)
-                        training_context['optimizer'].adjust_learning_rate(new_lr,True)
+                if self.monitor_op(current, self.best):
+                    self.best = current
+                    self.wait = 0
+                elif not self.in_cooldown():
+                    self.wait += 1
+                    if self.wait >= self.patience:
+                        old_lr = float(training_context['optimizer'].lr)
+                        if old_lr > self.min_lr:
+                            new_lr = old_lr * self.factor
+                            new_lr = max(new_lr, self.min_lr)
+                            training_context['optimizer'].adjust_learning_rate(new_lr,True)
 
-                        if self.verbose > 0:
-                            print('\nEpoch %05d: ReduceLROnPlateau reducing '
-                                  'learning rate to %s.' % (training_context['current_epoch'] + 1, new_lr))
-                        self.cooldown_counter = self.cooldown
-                        self.wait = 0
+                            if self.verbose > 0:
+                                print('\nEpoch %05d: ReduceLROnPlateau reducing '
+                                      'learning rate to %s.' % (training_context['current_epoch'] + 1, new_lr))
+                            self.cooldown_counter = self.cooldown
+                            self.wait = 0
 
     def on_batch_end(self, training_context):
-        if training_context['current_batch']>0 and training_context['current_batch']%100==0:
+        if self.unit_base is None:
+            if training_context['total_batch']>_session.epoch_equivalent:
+                self.unit_base='batch'
+                print('one epoch have {0} batches, use {1} as epoch equivalent in long epoch. '.format(training_context['total_batch'],_session.epoch_equivalent))
+            else:
+                self.unit_base = 'epoch'
+                print('ReduceLROnPlateau reseted.')
+
+        num_batches = training_context['current_epoch'] * training_context['total_batch'] + training_context[ 'current_batch']
+        if self.unit_base=='batch' and training_context['current_batch']>0 and training_context['current_batch']%_session.epoch_equivalent==0:
             training_context['current_lr']=training_context['optimizer'].lr
-            current =to_numpy(training_context['losses'].get(self.monitor,training_context['metrics'].get(self.monitor,training_context['losses']['total_losses']))).mean()
+            history=training_context['losses'].get(self.monitor,training_context['metrics'].get(self.monitor,training_context['losses']['total_losses']))
+            current =to_numpy(history[-min(5,len(history)):]).mean()
             if current is None:
                 warnings.warn(
                     'Reduce LR on plateau conditioned on metric `%s` '
