@@ -9,11 +9,11 @@ from ..backend.load_backend import get_backend
 from ..backend.pillow_backend import image2array
 from ..misc.ipython_utils import is_in_ipython, is_in_colab
 from ..data.mask_common import label2color
-
-if is_in_ipython():
+from ..misc.visualization_utils import *
+if is_in_ipython() or is_in_colab():
     from IPython import display
 
-from ..misc.visualization_utils import *
+
 
 _session = get_session()
 _backend = get_backend()
@@ -72,15 +72,20 @@ class TileImageCallback(VisualizationCallbackBase):
         data = training_context['train_data']
         model = training_context['current_model']
 
-        for data_key in data.key_list:
-            if data_key == data_feed[model.signature.key_list[0]]:
-                input = data[data_feed[model.signature.key_list[0]]]
-            elif 'output' in data_key and not 'target' in data_key and data_key not in data_feed.value_list:
-                    output = data[data_key]
-            elif ('target' in data_key or 'label' in data_key or 'mask' in data_key) and not 'output' in data_key and data_key in data_feed.value_list:
-                target = data[data_key]
-        if output is None and 'tensor' in model.__class__.__name__.lower():
-            output=model.clone()
+        if len(data_feed) == len(data) :
+            input = data.value_list[0]
+            target =data.value_list[1]
+            output =data.value_list[2]
+        else:
+            for data_key in data.key_list:
+                if data_key == data_feed[model.signature.key_list[0]]:
+                    input = data[data_feed[model.signature.key_list[0]]]
+                elif 'output' in data_key and not 'target' in data_key and data_key not in data_feed.value_list:
+                        output = data[data_key]
+                elif ('target' in data_key or 'label' in data_key or 'mask' in data_key) and not 'output' in data_key and data_key in data_feed.value_list:
+                    target = data[data_key]
+            if output is None and 'tensor' in model.__class__.__name__.lower():
+                output=model.clone()
 
         if self.include_input and input is not None:
             if self.reverse_image_transform is not None:
