@@ -415,10 +415,10 @@ def load_lfw(kind='train', is_flatten=None, is_onehot=None):
     return dataset
 
 
-def load_examples_data(dataset_name,include_test=False):
+def load_examples_data(dataset_name):
     dataset_name = dataset_name.strip().lower()
     if dataset_name.lower() not in ['pokemon', 'hanzi', 'animals', 'nsfw', 'simpsons', 'horse2zebra', 'people',
-                                    'autodrive']:
+                                    'autodrive','superresolution']:
         raise ValueError('Not a  valid  dataset_name.')
     dataset_name = 'examples_' + dataset_name
     dirname = os.path.join(_trident_dir, dataset_name)
@@ -445,11 +445,11 @@ def load_examples_data(dataset_name,include_test=False):
         extract_archive(train_imgs_path, dirname, archive_format='tar')
         dataset = load_folder_images(dataset_name, os.path.join(dirname, 'train'), folder_as_label=True,
                                      expect_data_type=ExpectDataType.gray)
-        if include_test:
-            dataset_test = load_folder_images(dataset_name, os.path.join(dirname, 'test'), folder_as_label=True,
-                                              expect_data_type=ExpectDataType.gray)
 
-            dataset.testdata = dataset_test.traindata
+        dataset_test = load_folder_images(dataset_name, os.path.join(dirname, 'test'), folder_as_label=True,
+                                          expect_data_type=ExpectDataType.gray)
+
+        dataset.testdata = dataset_test.traindata
         dataset.class_names['zh-cn'] = dataset.class_names['en-us']
         return dataset
 
@@ -552,6 +552,19 @@ def load_examples_data(dataset_name,include_test=False):
 
         dataset = DataProviderV2(dataset_name=dataset_name, traindata=Iterator(data=imgdata, label=mskdata))
         print('get autodrive images :{0}'.format(len(dataset)))
+        return dataset
+    elif dataset_name == 'examples_superresolution':
+        download_file_from_google_drive('1v1uoymrWI_MLSiGvSGW7tWJYSnzzXpEQ', dirname, 'superresolution.tar')
+        train_imgs_path = os.path.join(dirname, 'superresolution.tar')
+        extract_archive(train_imgs_path, dirname, archive_format='tar')
+        imgs = glob.glob(os.path.join(dirname,  '*.*g'))
+        imgs.extend(glob.glob(os.path.join(dirname, '*.bmp')))
+        print('get super resolution images :{0}'.format(len(imgs)))
+        imgdata = ImageDataset(images=imgs*2, expect_data_type=ExpectDataType.rgb,symbol='lr')
+        labeldata = ImageDataset(images=imgs*2, expect_data_type=ExpectDataType.rgb,symbol='hr')
+        dataset = DataProviderV2(dataset_name=dataset_name, traindata=Iterator(data=imgdata,label=labeldata))
+
+
         return dataset
 
     else:
