@@ -3,19 +3,28 @@ import itertools
 import sys
 import string
 import random
+import hashlib
+import pickle
+import time
+import threading
 from typing import List, TypeVar, Iterable, Tuple, Union
 from enum import Enum, unique
 import numpy as np
 from skimage import color
-from ..backend.common import DataSpec, PrintException,OrderedDict
+from ..backend.common import DataSpec, PrintException, OrderedDict
 from .image_common import gray_scale, image2array, mask2array, image_backend_adaptive, reverse_image_backend_adaptive, \
     unnormalize, array2image, ExpectDataType, GetImageMode
 from .label_common import label_backend_adaptive
-from .mask_common import mask_backend_adaptive,color2label
-from .bbox_common import xywh_to_xyxy,xyxy_to_xywh
+from .mask_common import mask_backend_adaptive, color2label
+from .bbox_common import xywh_to_xyxy, xyxy_to_xywh
 from .samplers import *
 from ..backend.load_backend import get_backend
 
+try:
+    import Queue
+except ImportError:
+    import queue as Queue
+
 if get_backend() == 'pytorch':
     from ..backend.pytorch_backend import to_numpy, to_tensor, ReplayBuffer
     import torch
@@ -103,7 +112,9 @@ class ImageDataset(Dataset):
             return image_backend_adaptive(img_data)
         if isinstance(img_data, np.ndarray):
             for fc in self.image_transform_funcs:
-                if not fc.__qualname__.startswith('random_') or  'crop' in fc.__qualname__  or  'rescale' in fc.__qualname__  or  (fc.__qualname__.startswith('random_') and random.randint(0,10)%2==0):
+                if not fc.__qualname__.startswith(
+                        'random_') or 'crop' in fc.__qualname__ or 'rescale' in fc.__qualname__ or (
+                        fc.__qualname__.startswith('random_') and random.randint(0, 10) % 2 == 0):
                     img_data = fc(img_data)
             img_data = image_backend_adaptive(img_data)
 
@@ -137,15 +148,16 @@ class ImageDataset(Dataset):
 
 
 class MaskDataset(Dataset):
-    def __init__(self, masks=None,class_names=None, expect_data_type: ExpectDataType = ExpectDataType.label_mask,
+    def __init__(self, masks=None, class_names=None, expect_data_type: ExpectDataType = ExpectDataType.label_mask,
                  get_image_mode: GetImageMode = GetImageMode.processed, symbol=None, name=''):
         super().__init__(symbol=symbol, expect_data_type=expect_data_type, name=name)
-        if expect_data_type not in [ExpectDataType.label_mask, ExpectDataType.binary_mask,ExpectDataType.alpha_mask,ExpectDataType.color_mask]:
+        if expect_data_type not in [ExpectDataType.label_mask, ExpectDataType.binary_mask, ExpectDataType.alpha_mask,
+                                    ExpectDataType.color_mask]:
             raise ValueError('Only mask is valid expect image type. ')
 
         self.__add__(masks)
         self.get_image_mode = get_image_mode
-        self.palette=OrderedDict()
+        self.palette = OrderedDict()
         self.mask_transform_funcs = []
         self.is_pair_process = False
         self.class_names = {}
@@ -165,8 +177,8 @@ class MaskDataset(Dataset):
 
             if self.expect_data_type == ExpectDataType.binary_mask:
                 img = mask2array(img).astype(np.int64)
-                mv=img.max()
-                img[img== mv] = 255
+                mv = img.max()
+                img[img == mv] = 255
                 img[img <= 0] = 0
                 img[img > 128] = 255
             elif self.expect_data_type == ExpectDataType.alpha_mask:
@@ -174,19 +186,18 @@ class MaskDataset(Dataset):
             elif self.expect_data_type == ExpectDataType.label_mask:
                 if '.png' in img:
                     img = image2array(img)
-                    img=img[:,:,1]
+                    img = img[:, :, 1]
                 else:
                     img = mask2array(img).astype(np.int64)
 
             elif self.expect_data_type == ExpectDataType.color_mask:
-                img =image2array(img)
-                if img.ndim==2:
+                img = image2array(img)
+                if img.ndim == 2:
                     pass
-                elif img.ndim==3:
-                    img=img[:,:,:3]
-                    if len(self.palette)>0:
-                        img=color2label(img,self.palette).astype(np.int64)
-
+                elif img.ndim == 3:
+                    img = img[:, :, :3]
+                    if len(self.palette) > 0:
+                        img = color2label(img, self.palette).astype(np.int64)
 
         if self.get_image_mode == GetImageMode.raw:
             return img
@@ -206,12 +217,15 @@ class MaskDataset(Dataset):
 
     def mask_transform(self, mask_data):
         if len(self.mask_transform_funcs) == 0:
-            return mask_backend_adaptive(mask_data,label_mapping=self.class_names,expect_data_type=self.expect_data_type)
+            return mask_backend_adaptive(mask_data, label_mapping=self.class_names,
+                                         expect_data_type=self.expect_data_type)
         if isinstance(mask_data, np.ndarray):
             for fc in self.mask_transform_funcs:
-                if not fc.__qualname__.startswith('random_') or  'crop' in fc.__qualname__  or  'rescale' in fc.__qualname__  or  (fc.__qualname__.startswith('random_') and random.randint(0,10)%2==0):
+                if not fc.__qualname__.startswith(
+                        'random_') or 'crop' in fc.__qualname__ or 'rescale' in fc.__qualname__ or (
+                        fc.__qualname__.startswith('random_') and random.randint(0, 10) % 2 == 0):
                     mask_data = fc(mask_data)
-            #mask_data = mask_backend_adaptive(mask_data)
+            # mask_data = mask_backend_adaptive(mask_data)
             return mask_data
         else:
             return mask_data
@@ -326,15 +340,16 @@ class BboxDataset(Dataset):
 
 
 class SsdBboxDataset(BboxDataset):
-    def __init__(self, boxes=None, image_size=None, priors=None, center_variance=0.1, size_variance=0.2,gt_overlap_tolerance=0.5,
-                 expect_data_type=ExpectDataType.absolute_bbox, class_names=None, symbol='bbox', name=''):
+    def __init__(self, boxes=None, image_size=None, priors=None, center_variance=0.1, size_variance=0.2,
+                 gt_overlap_tolerance=0.5, expect_data_type=ExpectDataType.absolute_bbox, class_names=None,
+                 symbol='bbox', name=''):
         super().__init__(boxes=boxes, image_size=image_size, expect_data_type=expect_data_type, class_names=class_names,
                          symbol=symbol, name=name)
         self.priors = priors
         self.center_variance = center_variance
         self.size_variance = size_variance
         self.label_transform_funcs = []
-        self.gt_overlap_tolerance=gt_overlap_tolerance
+        self.gt_overlap_tolerance = gt_overlap_tolerance
         self.bbox_post_transform_funcs = []
 
     def area_of(self, left_top, right_bottom):
@@ -368,16 +383,14 @@ class SsdBboxDataset(BboxDataset):
         area1 = self.area_of(boxes1[..., :2], boxes1[..., 2:])
         return overlap_area / (area0 + area1 - overlap_area + eps)
 
-
     def convert_boxes_to_locations(self, center_form_boxes, center_form_priors):
         if len(center_form_priors.shape) + 1 == len(center_form_boxes.shape):
             center_form_priors = np.expand_dims(center_form_priors, 0)
-        return np.concatenate(
-            [(center_form_boxes[..., :2] - center_form_priors[..., :2]) / center_form_priors[..., 2:] / self.center_variance,
-             np.log(center_form_boxes[..., 2:] / center_form_priors[..., 2:]) / self.size_variance],
+        return np.concatenate([(center_form_boxes[..., :2] - center_form_priors[..., :2]) / center_form_priors[...,
+                                                                                            2:] / self.center_variance,
+                               np.log(center_form_boxes[..., 2:] / center_form_priors[..., 2:]) / self.size_variance],
             axis=len(center_form_boxes.shape) - 1)
 
-
     def assign_priors(self, gt_boxes, gt_labels, center_form_priors, iou_threshold):
         # """Assign ground truth boxes and targets to priors.
         #
@@ -408,18 +421,18 @@ class SsdBboxDataset(BboxDataset):
         # boxes = gt_boxes[best_target_per_prior_index]
         # return boxes, labels
         corner_form_priors = xywh_to_xyxy(center_form_priors)
-        ious = self.iou_of(np.expand_dims(gt_boxes,0), np.expand_dims(corner_form_priors,1))
+        ious = self.iou_of(np.expand_dims(gt_boxes, 0), np.expand_dims(corner_form_priors, 1))
         # size: num_priors
-        best_target_per_prior, best_target_per_prior_index =np.max(ious,axis=1),np.argmax(ious,axis=1)
+        best_target_per_prior, best_target_per_prior_index = np.max(ious, axis=1), np.argmax(ious, axis=1)
         # size: num_targets
-        best_prior_per_target, best_prior_per_target_index =np.max(ious,axis=0), np.argmax(ious,axis=0)
+        best_prior_per_target, best_prior_per_target_index = np.max(ious, axis=0), np.argmax(ious, axis=0)
         for target_index, prior_index in enumerate(best_prior_per_target_index):
             best_target_per_prior_index[prior_index] = target_index
         # 2.0 is used to make sure every target has a prior assigned
-        best_prior_per_target_index_list=best_prior_per_target_index.tolist()
+        best_prior_per_target_index_list = best_prior_per_target_index.tolist()
         for i in range(best_target_per_prior.shape[0]):
             if i in best_prior_per_target_index_list:
-                best_target_per_prior[i]=2
+                best_target_per_prior[i] = 2
         labels = gt_labels[best_target_per_prior_index]
         labels[best_target_per_prior < iou_threshold] = 0  # the backgournd id
         boxes = gt_boxes[best_target_per_prior_index]
@@ -438,12 +451,12 @@ class SsdBboxDataset(BboxDataset):
         if bbox is None or len(bbox) == 0:
             return np.zeros((self.priors.shape[0], 4)).astype(np.float32), np.zeros((self.priors.shape[0])).astype(
                 np.int64)
-        elif isinstance(bbox,np.ndarray):
+        elif isinstance(bbox, np.ndarray):
             height, width = self.image_size
-            bbox[:, 0] = bbox[:, 0]/width
-            bbox[:, 2] = bbox[:, 2]/width
-            bbox[:, 1] = bbox[:, 1]/height
-            bbox[:, 3] = bbox[:, 3] /height
+            bbox[:, 0] = bbox[:, 0] / width
+            bbox[:, 2] = bbox[:, 2] / width
+            bbox[:, 1] = bbox[:, 1] / height
+            bbox[:, 3] = bbox[:, 3] / height
             if bbox.shape[-1] == 5:
                 gt_box = bbox[:, :4]
                 gt_label = bbox[:, 4]
@@ -455,7 +468,6 @@ class SsdBboxDataset(BboxDataset):
                 return bbox
 
 
-
 class NumpyDataset(Dataset):
     def __init__(self, data=None, expect_data_type=ExpectDataType.array_data, symbol=None, name=''):
         super().__init__(symbol=symbol, expect_data_type=expect_data_type, name=name)
@@ -494,11 +506,15 @@ class Iterator(object):
         self._data = NumpyDataset()
         self._label = LabelDataset()
         self._unpair = NumpyDataset()
+        self.workers = 2
+        self.itr = 0
         if data is not None and isinstance(data, Dataset):
             self._data = data
         if label is not None and isinstance(label, Dataset):
             self._label = label
-            if isinstance(self._label, (MaskDataset, ImageDataset, BboxDataset)) and isinstance(self._data, ImageDataset) and len(self._label) == len(self._data):
+            if isinstance(self._label, (MaskDataset, ImageDataset, BboxDataset)) and isinstance(self._data,
+                                                                                                ImageDataset) and len(
+                    self._label) == len(self._data):
                 self._label.is_pair_process = self._data.is_pair_process = self.is_pair_process = True
             else:
                 self._label.is_pair_process = self._data.is_pair_process = self.is_pair_process = False
@@ -510,8 +526,8 @@ class Iterator(object):
         self.paired_transform_funcs = []
         self.batch_sampler = BatchSampler(self, self._minibatch_size, is_shuffle=True, drop_last=False)
         self._sample_iter = iter(self.batch_sampler)
-
-
+        self.buffer_size = 10
+        self.out_queue = Queue.Queue(maxsize=self.buffer_size)
 
     @property
     def data(self):
@@ -522,7 +538,7 @@ class Iterator(object):
         self._data = value
         if self._label is not None and isinstance(self._label, (MaskDataset, ImageDataset)) and isinstance(self._data,
                                                                                                            ImageDataset) and len(
-                self._label) == len(self._data):
+            self._label) == len(self._data):
             self._label.is_pair_process = self._data.is_pair_process = self.is_pair_process = True
         else:
             self._label.is_pair_process = self._data.is_pair_process = self.is_pair_process = False
@@ -537,7 +553,9 @@ class Iterator(object):
     @label.setter
     def label(self, value):
         self._label = value
-        if isinstance(self._label, (MaskDataset, ImageDataset,BboxDataset)) and isinstance(self._data, ImageDataset) and len(self._label) == len(self._data):
+        if isinstance(self._label, (MaskDataset, ImageDataset, BboxDataset)) and isinstance(self._data,
+                                                                                            ImageDataset) and len(
+                self._label) == len(self._data):
             self._label.is_pair_process = self._data.is_pair_process = self.is_pair_process = True
         else:
             self._label.is_pair_process = self._data.is_pair_process = self.is_pair_process = False
@@ -557,7 +575,8 @@ class Iterator(object):
 
     @property
     def palette(self):
-        if isinstance(self._label,MaskDataset) and self._label.expect_data_type in [ExpectDataType.label_mask,ExpectDataType.color_mask]:
+        if isinstance(self._label, MaskDataset) and self._label.expect_data_type in [ExpectDataType.label_mask,
+                                                                                     ExpectDataType.color_mask]:
             return self._label.palette
         else:
             return None
@@ -575,12 +594,12 @@ class Iterator(object):
     def update_signature(self, arg_names):
         iterdata = self.next()
         if self.signature is None:
-            self.signature =OrderedDict()
+            self.signature = OrderedDict()
             for i in range(len(arg_names)):
-                arg=arg_names[i]
-                data=iterdata[i][0]
-                self.signature[arg]=data.shape
-        elif len(self.signature.key_list)==len(arg_names):
+                arg = arg_names[i]
+                data = iterdata[i][0]
+                self.signature[arg] = data.shape
+        elif len(self.signature.key_list) == len(arg_names):
             self.signature = OrderedDict()
             for i in range(len(arg_names)):
                 arg = arg_names[i]
@@ -593,7 +612,7 @@ class Iterator(object):
             raise ValueError('data deed and arg_names should be the same length')
         else:
             self.signature = OrderedDict()
-            iterdata=self.next()
+            iterdata = self.next()
             for i in range(len(arg_names)):
                 arg = arg_names[i]
                 data = iterdata[i][0]
@@ -621,11 +640,19 @@ class Iterator(object):
             return img_data, paired_img
 
     def __getitem__(self, index: int):
+        #start = time.time()
+
         try:
             bbox = None
             mask = None
             data = self.data.__getitem__(index % len(self.data)) if len(self.data) > 0 else None
+            # stop = time.time()
+            # print('get data:{0}'.format(stop - start))
+            # start=stop
             label = self.label.__getitem__(index % len(self.label)) if len(self.label) > 0 else None
+            # stop = time.time()
+            # print('get label:{0}'.format(stop - start))
+            # start = stop
             if isinstance(self.label, BboxDataset):
                 data, label = self.paired_transform(data, label)
                 if hasattr(self.data, 'image_transform'):
@@ -637,7 +664,7 @@ class Iterator(object):
                     else:
                         bbox = new_label
                 else:
-                    bbox =label
+                    bbox = label
             elif isinstance(self.label, MaskDataset):
                 data, label = self.paired_transform(data, label)
                 if hasattr(self.data, 'image_transform'):
@@ -651,46 +678,73 @@ class Iterator(object):
                 else:
                     mask = label.copy()
                 label = None
+            elif isinstance(self.label, ImageDataset):
+                data, label = self.paired_transform(data, label)
+                # stop = time.time()
+                # print('paired_transform:{0}'.format(stop - start))
+                # start = stop
+                if hasattr(self.data, 'image_transform'):
+                    data = self.data.image_transform(data)
+                    # stop = time.time()
+                    # print('data image_transform:{0}'.format(stop - start))
+                    # start = stop
+                if hasattr(self.label, 'image_transform'):
+                    label = self.label.image_transform(label)
+                    # stop = time.time()
+                    # print('label image_transform:{0}'.format(stop - start))
+                    # start = stop
             else:
                 if hasattr(self.label, 'label_transform'):
                     label = self.label.label_transform(label)
 
-
-
-            if hasattr(self.label, 'label_transform') and not isinstance(self.label, (BboxDataset,MaskDataset)):
+            if hasattr(self.label, 'label_transform') and not isinstance(self.label, (BboxDataset, MaskDataset)):
                 label = self.label.label_transform(label)
 
             unpair = self.unpair.__getitem__(index % len(self.unpair)) if len(self.unpair) > 0 else None
 
             return_data = []
-            if self.signature is None or len(self.signature)==0:
-                self.signature=OrderedDict()
+            if self.signature is None or len(self.signature) == 0:
+                self.signature = OrderedDict()
                 if data is not None:
-                    self.signature['data' if self.data.symbol is None or len(self.data.symbol)==0 else self.data.symbol]=data.shape
+                    self.signature['data' if self.data.symbol is None or len(
+                        self.data.symbol) == 0 else self.data.symbol] = data.shape
                 if bbox is not None:
-                    self.signature['bbox' if self.label.symbol is None or len(self.label.symbol) ==0 else self.label.symbol] =  bbox.shape
+                    self.signature['bbox' if self.label.symbol is None or len(
+                        self.label.symbol) == 0 else self.label.symbol] = bbox.shape
                 if mask is not None:
-                    self.signature['mask' if self.label.symbol is None or len(self.label.symbol) ==0 else self.label.symbol] = mask.shape
+                    self.signature['mask' if self.label.symbol is None or len(
+                        self.label.symbol) == 0 else self.label.symbol] = mask.shape
                 if label is not None:
-                    self.signature['label'] = label.shape if  isinstance(label,np.ndarray) else type( label)
+                    self.signature['label' if self.label.symbol is None or len(
+                        self.label.symbol) == 0 else self.label.symbol] = label.shape if isinstance(label,
+                                                                                                    np.ndarray) else \
+                        type(
+                        label)
                 if unpair is not None:
-                    self.signature[ 'unpair' if self.unpair.symbol is  None or len(self.unpair.symbol)==0 else self.unpair.symbol] = unpair.shape
-
+                    self.signature['unpair' if self.unpair.symbol is None or len(
+                        self.unpair.symbol) == 0 else self.unpair.symbol] = unpair.shape
+                # stop = time.time()
+                # print('signature:{0}'.format(stop - start))
+                # start = stop
 
             if data is not None:
                 return_data.append(data)
+            if label is not None:
+                return_data.append(label)
             if bbox is not None:
                 return_data.append(bbox)
             if mask is not None:
                 return_data.append(mask)
-            if label is not None:
-                return_data.append(label)
             if unpair is not None:
                 return_data.append(unpair)
+            # stop = time.time()
+            # print('prepare tuple:{0}'.format(stop - start))
+            # start = stop
             return tuple(return_data)
         except:
             PrintException()
 
+
     def _next_index(self):
         return next(self._sample_iter)
 
@@ -698,11 +752,37 @@ class Iterator(object):
         return self._sample_iter
 
     def next(self):
-        return next(self._sample_iter)
+        if self.out_queue.qsize() == 0:
+            in_data = self._sample_iter.__next__()
+            self.out_queue.put(in_data,False)
+            #self.out_queue.task_done()
+
+        out_data = self.out_queue.get(False)
+        #self.out_queue.task_done()
+
+        if self.out_queue.qsize() <= self.buffer_size // 2:
+            for i in range(2):
+                in_data = self._sample_iter.__next__()
+                self.out_queue.put(in_data,False)
+                #self.out_queue.task_done()
+        return out_data
 
     def __next__(self):
-        return next(self._sample_iter)
+        if self.out_queue.qsize() == 0:
+            in_data = self._sample_iter.__next__()
+            self.out_queue.put(in_data,False)
+            #self.out_queue.task_done()
+
+        out_data = self.out_queue.get(False)
+        #self.out_queue.task_done()
+        yield out_data
+        if self.out_queue.qsize() <= self.buffer_size // 2:
+            for i in range(self.buffer_size -self.out_queue.qsize()):
+                in_data = self._sample_iter.__next__()
+                self.out_queue.put(in_data,False)
+                #self.out_queue.task_done()
+
+
 
     def __len__(self):
         return max([len(self.data) if self.data is not None else 0, len(self.unpair) if self.unpair is not None else 0])
-
