@@ -4,6 +4,7 @@ from __future__ import print_function
 import six
 import os
 import random
+import time
 import re
 import numpy as np
 import math
@@ -17,8 +18,8 @@ from ..backend.common import *
 from itertools import repeat
 import cv2
 
-__all__ = ['read_image','read_mask','save_image','save_mask','image2array','array2image','mask2array','array2mask','list_pictures','normalize','unnormalize','channel_reverse','blur','random_blur','random_crop','resize','rescale','add_noise','gray_scale','to_rgb','auto_level','random_invert_color',
-           'image_backend_adaptive','reverse_image_backend_adaptive','random_adjust_hue', 'random_channel_shift', 'random_cutout','random_rescale_crop','random_center_crop','adjust_gamma','random_adjust_gamma','adjust_contast','random_adjust_contast','clahe','erosion_then_dilation','image_erosion','image_dilation','adaptive_binarization','random_transform',
+__all__ = ['read_image','read_mask','save_image','save_mask','image2array','array2image','mask2array','array2mask','list_pictures','normalize','unnormalize','channel_reverse','blur','random_blur','random_crop','resize','rescale','downsample_then_upsample','add_noise','gray_scale','to_rgb','auto_level','random_invert_color',
+           'image_backend_adaptive','reverse_image_backend_adaptive','random_adjust_hue', 'random_channel_shift', 'random_cutout','random_rescale_crop','random_center_crop','adjust_gamma','random_adjust_gamma','adjust_contast','random_adjust_contast','clahe','erosion_then_dilation','dilation_then_erosion','image_erosion','image_dilation','adaptive_binarization','random_transform',
            'horizontal_flip','random_mirror']
 
 
@@ -249,6 +250,15 @@ def rescale(scale,order=1):
     img_op.scale=(scale,scale)
     return img_op
 
+def downsample_then_upsample(scale=4,order=1,repeat=1):
+    def img_op(image: np.ndarray):
+        for i in range(repeat):
+            image=rescale(scale=1.0/scale)(image)
+            image = rescale(scale=scale)(image)
+        return image
+    return img_op
+
+
 def invert_color():
     def img_op(image:np.ndarray):
         if np.min(image)>=0 and np.max(image)<=1:
@@ -361,14 +371,13 @@ def random_adjust_hue(hue_range=(-20,20),saturation_range=(-10,10),lightness_ran
 
 def auto_level():
     def img_op(image: np.ndarray):
-        counts, bins = np.histogram(image.reshape([-1]), np.arange(0, 255, 32).tolist())
-        if 0.6<counts[:1].sum()/counts.sum()<0.99:
-            image[image<image.mean()//2]=0
-        if 0.7<counts[-3:].sum() / counts.sum() <0.99:
-            image[image>(256-(256-image.mean())//2) ] = 255
+        minv=np.percentile(image,5)
+        maxv = np.percentile(image, 95)
+        if maxv-minv<40:
+            minv =image.min()
+            maxv = image.max()
 
-        image=image-np.clip(np.min(image,keepdims=True),0,255)
-        image=np.clip(image*255/(np.max(image,keepdims=True)+1e-5),0,255)
+        image = np.clip((image - minv) * (255.0 / (maxv - minv)),0,255)
         return image.astype(np.float32)
     return img_op
 
@@ -422,8 +431,10 @@ def random_crop(h, w):
 
 
 def random_rescale_crop(h,w,scale=(0.5,2),order=1):
+
     scalemin, scalemax = scale
     def img_op(*image:np.ndarray):
+        #start = time.time()
         results = to_list(image)
         scale=np.random.choice(np.random.uniform(scalemin,scalemax,100))
         height, width = image[0].shape[:2]
@@ -437,6 +448,9 @@ def random_rescale_crop(h,w,scale=(0.5,2),order=1):
             offset_y = random.choice(range(height-h))
         offset_x1 = random.choice(range(w-width)) if w>width else 0
         offset_y1 = random.choice(range(h-height)) if h>height else 0
+        # stop = time.time()
+        # print('prepare random crop:{0}'.format(stop - start))
+        # start = stop
         for i in range(len(results)):
             im = results[i]
             if im is None:
@@ -471,13 +485,20 @@ def random_rescale_crop(h,w,scale=(0.5,2),order=1):
                 returnData[offset_y1:offset_y1+crop_im.shape[0], offset_x1:offset_x1+crop_im.shape[1]] = crop_im
                 results[i] = returnData
             elif im.ndim==3:
-                im = transform.rescale(im, (scale, scale), clip=False, anti_aliasing=True,
-                                       multichannel=True if len(im.shape) == 3 else False,
-                                       order=0 if im.ndim == 2 else order)
+                # im = transform.rescale(im, (scale, scale), clip=False, anti_aliasing=True,
+                #                        multichannel=True if len(im.shape) == 3 else False,
+                #                        order=0 if im.ndim == 2 else order)
+                im=cv2.resize(im,None,fx=scale,fy=scale,interpolation=cv2.INTER_AREA)
+                # stop = time.time()
+                # print('resize:{0}'.format(stop - start))
+                # start = stop
                 returnData =np.zeros((h,w,3),dtype=np.float32)
                 crop_im=im[offset_y:offset_y+h,offset_x:offset_x+w,:]
                 returnData[offset_y1:offset_y1+crop_im.shape[0],offset_x1:offset_x1+crop_im.shape[1],:]=crop_im
                 results[i]=returnData
+                # stop = time.time()
+                # print('crop:{0}'.format(stop - start))
+                # start = stop
         return unpack_singleton(tuple(results))
     return img_op
 
@@ -652,13 +673,7 @@ def clahe(clip_limit=0.1,nbins=16):
         return image*255.0
     return img_op
 
-def erosion_then_dilation(filter_size=3,repeat=1):
-    def img_op(image: np.ndarray):
-        for i in range(repeat):
-            image=morphology.erosion(image,morphology.square(filter_size))
-            image = morphology.dilation(image, morphology.square(filter_size))
-        return image
-    return img_op
+
 
 def image_erosion(filter_size=3,repeat=1):
     def img_op(image: np.ndarray):
@@ -674,11 +689,29 @@ def image_dilation(filter_size=3,repeat=1):
         return image
     return img_op
 
+
+def erosion_then_dilation(filter_size=3,repeat=1):
+    def img_op(image: np.ndarray):
+        for i in range(repeat):
+            image=morphology.erosion(image,morphology.square(filter_size))
+            image = morphology.dilation(image, morphology.square(filter_size))
+        return image
+    return img_op
+
+def dilation_then_erosion(filter_size=3,repeat=1):
+    def img_op(image: np.ndarray):
+        for i in range(repeat):
+            image=morphology.dilation(image,morphology.square(filter_size))
+            image = morphology.erosion(image, morphology.square(filter_size))
+        return image
+    return img_op
+
+
 def adaptive_binarization(threshold_type='otsu' ):
     def img_op(image: np.ndarray):
         image=image.astype(np.float32)
         local_thresh=None
-        blur = gaussian(image, sigma=0.2)
+        blur = gaussian(image, sigma=0.5,multichannel=True if image.ndim==3 else None)
         try:
             if threshold_type=='otsu'  :
                 if len(blur.shape) > 2 and blur.shape[-1] in (3, 4):
@@ -693,7 +726,22 @@ def adaptive_binarization(threshold_type='otsu' ):
                 local_thresh=threshold_local(blur, block_size=35, offset=10)
             elif threshold_type=='isodata':
                 local_thresh=threshold_isodata(blur, nbins=256)
-            image =(image > local_thresh).astype(np.float32)*255.0
+            elif threshold_type == 'percentile':
+                p10=np.percentile(image.copy(),10)
+                p90 = np.percentile(image.copy(), 90)
+                if abs(image.mean()-p90)<abs(image.mean()-p10) and p90-p10>80:#white background
+                    image[image<p10]=0
+                    image[image > p90] = 255
+                elif abs(image.mean() - p90) > abs(image.mean() - p10)and p90-p10>80:  # white background
+                    image[image > p90] = 255
+                    image[image < p10] = 0
+                return image
+
+            blur =(blur > local_thresh).astype(np.float32)*255.0
+            if blur.max()-blur.min()<10 :
+                pass
+            else:
+                image=blur
         except Exception as e:
             print(e)
         return image
@@ -709,13 +757,13 @@ def blur(sigma=0.3):
         return image
     return img_op
 
-def random_blur():
+def random_blur(sigma=(0.1,0.6)):
     def img_op(image: np.ndarray):
         n=random.randint(0,3)
         if n==0:
             return image
         else:
-            image =  gaussian(image.astype(np.float32), sigma=np.random.choice(np.arange(0.1,0.6,0.05)),multichannel=True)
+            image =  gaussian(image.astype(np.float32), sigma=np.random.choice(np.arange(sigma[0],sigma[1],0.05)),multichannel=True)
         return image
     return img_op
 
