@@ -11,7 +11,7 @@ import math
 from ..backend.common import *
 
 
-__all__ = ['label_backend_adaptive','get_onehot']
+__all__ = ['label_backend_adaptive','get_onehot','check_is_onehot']
 
 
 _session=get_session()
