@@ -193,7 +193,20 @@ def download_model_from_google_drive(file_id, dirname, filename=None, md5=None):
     if not filename:
         filename = file_id
     fpath = os.path.join(dirname, filename)
-    check_internet = is_connected()
+    isload = False
+    models_md5 = None
+    need_download = True
+    if os.path.exists(os.path.join(dirname, 'models_md5.json')):
+        os.remove(os.path.join(dirname, 'models_md5.json'))
+    try:
+        isload = download_file_from_google_drive('1AujFsfyzyYUcUrNI_GT1dvNrHaXIQkGz', dirname, 'models_md5.json')
+        check_internet = True
+        if os.path.exists(os.path.join(dirname, 'models_md5.json')):
+            with open(os.path.join(dirname, 'models_md5.json')) as f:
+                models_md5 = json.load(f)
+    except Exception:
+        check_internet=False
+
     if check_internet == False:
         if os.path.exists(os.path.join(dirname, filename)):
             print('internet connect  error,model file is already existing, donnot need download again.')
@@ -201,28 +214,8 @@ def download_model_from_google_drive(file_id, dirname, filename=None, md5=None):
         else:
             raise NewConnectionError
     else:
-        isload = False
-        models_md5 = None
-        need_download = True
-        if os.path.exists(os.path.join(dirname, 'models_md5.json')):
-            with open(os.path.join(dirname, 'models_md5.json')) as f:
-                models_md5 = json.load(f)
-            os.remove(os.path.join(dirname, 'models_md5.json'))
-        try:
-            isload = download_file_from_google_drive('1AujFsfyzyYUcUrNI_GT1dvNrHaXIQkGz', dirname, 'models_md5.json')
-            models_md5 = OrderedDict()
-            need_download = True
-            if isload == False:
-                if os.path.exists(os.path.join(dirname, filename)):
-                    stderr.write('download error, use local exsiting model')
-                else:
-                    stderr.write('sorry, download failed, cannot get the pretrained model.')
-            else:
-                need_download = True
-                if os.path.exists(os.path.join(dirname, 'models_md5.json')):
-                    with open(os.path.join(dirname, 'models_md5.json')) as f:
-                        models_md5 = json.load(f)
 
+        try:
             if os.path.exists(os.path.join(dirname, filename)):
                 if check_integrity(os.path.join(dirname, filename), models_md5[filename]):
                     need_download = False
@@ -243,7 +236,7 @@ def download_model_from_google_drive(file_id, dirname, filename=None, md5=None):
                     if token:
                         params = {'id': file_id, 'confirm': token}
                         response = session.get(url, params=params, stream=True)
-                        _save_response_content(response, fpath)
+                    _save_response_content(response, fpath)
         except Exception as e:
             print(e)
             PrintException()
