@@ -1,28 +1,30 @@
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
-import math
+
 import inspect
-import numpy as np
-from collections import *
-from functools import partial
+import math
 import uuid
-from copy import copy, deepcopy
+from collections import *
 from collections import deque
+from copy import copy, deepcopy
+from functools import partial
+from itertools import repeat
+
+import numpy as np
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
+from torch._six import container_abcs
 from torch.nn import init
 from torch.nn.parameter import Parameter
-from torch._six import container_abcs
-from itertools import repeat
 
+from .pytorch_activations import get_activation, Identity
+from .pytorch_layers import *
+from .pytorch_normalizations import get_normalization, SpectralNorm
+from .pytorch_pooling import *
 from ..backend.common import *
 from ..backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential
-from .pytorch_layers import *
-from .pytorch_pooling import *
-from .pytorch_activations import get_activation, Identity
-from .pytorch_normalizations import get_normalization, SpectralNorm
 
 __all__ = ['Conv2d_Block', 'Conv1d_Block', 'DepthwiseConv2d_Block', 'SeparableConv2d_Block', 'GcdConv2d_Block',
            'TransConv2d_Block', 'GcdConv2d_Block_1', 'Classifier1d', 'ShortCut2d', 'ConcateBlock', 'SqueezeExcite']
@@ -156,6 +158,7 @@ class Conv2d_Block(Layer):
 
             if self.use_spectral:
                 self.conv = nn.utils.spectral_norm(conv)
+                self.norm=None
             else:
                 self.conv = conv
             # if self.norm is not None:
@@ -399,21 +402,20 @@ class SeparableConv2d_Block(Layer):
 
 
 class GcdConv2d_Block(Layer):
-    def __init__(self, kernel_size=(3, 3), num_filters=32, strides=1, auto_pad=True, divisor_rank=0, activation='relu6',
-                 normalization=None, init=None, use_bias=False, init_bias=0, dilation=1, groups=1, add_noise=False,
-                 noise_intensity=0.005, dropout_rate=0, weights_contraint=None):
+    def __init__(self, kernel_size=(3, 3), num_filters=None, strides=1, auto_pad=True, padding_mode='zero', divisor_rank=0, activation=None,
+                 normalization=None, use_spectral=False, use_bias=False,  dilation=1, groups=1, add_noise=False,
+                 noise_intensity=0.005, dropout_rate=0, name=None, depth_multiplier=None, **kwargs ):
         super(GcdConv2d_Block, self).__init__()
         self.kernel_size = kernel_size
         self.num_filters = num_filters
         self.strides = _pair(strides)
         self.auto_pad = auto_pad
-
-        self.init = init
+        self.padding_mode=padding_mode
+        self.use_spectral=use_spectral
         self.use_bias = use_bias
-        self.init_bias = init_bias
+
         self.dilation = dilation
         self.groups = groups
-        self.weights_contraint = weights_contraint
         self.add_noise = add_noise
         self.noise_intensity = noise_intensity
         self.dropout_rate = dropout_rate
@@ -431,9 +433,14 @@ class GcdConv2d_Block(Layer):
                              use_bias=self.use_bias, init_bias=0, divisor_rank=self.divisor_rank,
                              dilation=self.dilation).to(self.device)
             conv.input_shape = input_shape
-            self._modules['conv'] = conv
-            self.conv = conv
-            output_shape = self._input_shape.clone().tolist()
+
+            if self.use_spectral:
+                self.conv = nn.utils.spectral_norm(conv)
+                self.norm=None
+            else:
+                self.conv = conv
+
+            output_shape = self._input_shape.clone()
             output_shape[0] = self.num_filters
             if self.norm != None:
                 self.norm.input_shape = output_shape
@@ -448,7 +455,7 @@ class GcdConv2d_Block(Layer):
         # dynamic generation
         x = self.conv(x)
         if self.activation is not None:
-            x = self.activation()(x)
+            x = self.activation(x)
 
         if self.dropout_rate > 0:
             x = F.dropout(x, p=self.dropout_rate, training=self.training)
