@@ -322,13 +322,13 @@ def get_normalization(fn_name):
     if fn_name is None:
         return None
     if isinstance(fn_name, str):
-        if fn_name.lower().strip() in ['instance','in','i']:
+        if fn_name.lower().strip() in ['instance_norm','instance','in','i']:
             return InstanceNorm()
-        elif  fn_name.lower().strip() in ['batch','b']:
+        elif  fn_name.lower().strip() in ['batch_norm','batch','bn','b']:
             return BatchNorm()
-        elif  fn_name.lower().strip() in ['group','g']:
+        elif  fn_name.lower().strip() in ['group_norm','group','gn','g']:
             return GroupNorm(num_groups=16)
-        elif fn_name.lower().strip() in ['spectral','spec','sp' ,'s']:
+        elif fn_name.lower().strip() in ['spectral_norm','spectral','spec','sp' ,'s']:
             return SpectralNorm()
     fn_modules = ['trident.layers.pytorch_normalizations']
     normalization_fn_ = get_function(fn_name, fn_modules)
