@@ -6,18 +6,26 @@ import collections
 import itertools
 import tensorflow as tf
 import tensorflow.keras.backend as K
+from tensorflow.python.eager import context
+from tensorflow.python.keras import initializers
+from tensorflow.python.framework import dtypes
 from tensorflow.python.keras.engine.base_layer import Layer
 from tensorflow_core.python.keras.utils import conv_utils
 from tensorflow.python.framework import tensor_shape
 from tensorflow.python.keras.engine.input_spec import InputSpec
 from tensorflow.python.keras.layers.convolutional import Conv
+from tensorflow.python.ops import gen_math_ops
+from tensorflow.python.ops import math_ops
+from tensorflow.python.ops import sparse_ops
 from tensorflow.python.ops import nn,nn_ops,array_ops
+from tensorflow.python.ops import standard_ops
 from tensorflow.python.client import device_lib
 from .tensorflow_activations import get_activation
+from tensorflow.python.keras.utils.generic_utils import deserialize_keras_object
+from tensorflow.python.keras.utils.generic_utils import serialize_keras_object
 from itertools import repeat
 import inspect
 
-
 from ..backend.common import *
 from ..backend.tensorflow_backend import *
 
@@ -94,8 +102,8 @@ class InputLayer(tf.keras.layers.InputLayer):
 
 
 class Dense(tf.keras.layers.Dense):
-    def __init__(self, output_filters, use_bias=True, activation=None,input_shape=None,name='', **kwargs ):
-        super(Dense, self).__init__(units=output_filters,use_bias=use_bias,activation=get_activation(activation),name=name,**kwargs)
+    def __init__(self, num_filters, use_bias=True, activation=None,input_shape=None,name='', **kwargs ):
+        super(Dense, self).__init__(units=num_filters,use_bias=use_bias,activation=get_activation(activation),name=name,**kwargs)
         inp_shape = kwargs.get('input_shape')
         if inp_shape is not None:
             self.input_spec = inp_shape
@@ -392,55 +400,7 @@ class Conv(Layer):
       op_padding = op_padding.upper()
     return op_padding
 
-class _ConvNd(Conv):
-  """Abstract N-D convolution layer (private, used as implementation base).
-  This layer creates a convolution kernel that is convolved
-  (actually cross-correlated) with the layer input to produce a tensor of
-  outputs. If `use_bias` is True (and a `bias_initializer` is provided),
-  a bias vector is created and added to the outputs. Finally, if
-  `activation` is not `None`, it is applied to the outputs as well.
-  Arguments:
-    rank: An integer, the rank of the convolution, e.g. "2" for 2D convolution.
-    filters: Integer, the dimensionality of the output space (i.e. the number
-      of filters in the convolution).
-    kernel_size: An integer or tuple/list of n integers, specifying the
-      length of the convolution window.
-    strides: An integer or tuple/list of n integers,
-      specifying the stride length of the convolution.
-      Specifying any stride value != 1 is incompatible with specifying
-      any `dilation_rate` value != 1.
-    padding: One of `"valid"`,  `"same"`, or `"causal"` (case-insensitive).
-    data_format: A string, one of `channels_last` (default) or `channels_first`.
-      The ordering of the dimensions in the inputs.
-      `channels_last` corresponds to inputs with shape
-      `(batch, ..., channels)` while `channels_first` corresponds to
-      inputs with shape `(batch, channels, ...)`.
-    dilation_rate: An integer or tuple/list of n integers, specifying
-      the dilation rate to use for dilated convolution.
-      Currently, specifying any `dilation_rate` value != 1 is
-      incompatible with specifying any `strides` value != 1.
-    activation: Activation function. Set it to None to maintain a
-      linear activation.
-    use_bias: Boolean, whether the layer uses a bias.
-    kernel_initializer: An initializer for the convolution kernel.
-    bias_initializer: An initializer for the bias vector. If None, the default
-      initializer will be used.
-    kernel_regularizer: Optional regularizer for the convolution kernel.
-    bias_regularizer: Optional regularizer for the bias vector.
-    activity_regularizer: Optional regularizer function for the output.
-    kernel_constraint: Optional projection function to be applied to the
-        kernel after being updated by an `Optimizer` (e.g. used to implement
-        norm constraints or value constraints for layer weights). The function
-        must take as input the unprojected variable and must return the
-        projected variable (which must have the same shape). Constraints are
-        not safe to use when doing asynchronous distributed training.
-    bias_constraint: Optional projection function to be applied to the
-        bias after being updated by an `Optimizer`.
-    trainable: Boolean, if `True` the weights of this layer will be marked as
-      trainable (and listed in `layer.trainable_weights`).
-    name: A string, the name of the layer.
-  """
-
+class _ConvNd(Layer):
   def __init__(self, rank,
                kernel_size,
                num_filters,
@@ -453,23 +413,47 @@ class _ConvNd(Conv):
                groups=1,
                transposed=False,
                name=None,
-               filter_rate=None,
+               depth_multiplier=None,
+               trainable=True,
                **kwargs
                ):
-    super(_ConvNd, self).__init__(rank=rank, filters=num_filters, kernel_size=kernel_size, strides=strides, padding='same' if auto_pad==True else 'valid', data_format='channels_last',
-        dilation_rate=dilation, activation=get_activation(activation), use_bias=use_bias, kernel_initializer='glorot_uniform', bias_initializer='zeros',
-        kernel_regularizer=None, bias_regularizer=None, activity_regularizer=None, kernel_constraint=None,
-        bias_constraint=None, trainable=True, name=name)
-    self.filter_rate=filter_rate
-    self.transposed=transposed
-    self.groups=groups
+    super(_ConvNd, self).__init__(trainable=trainable,
+        name=name,
+        **kwargs)
+    self.num_filters = None
+    if num_filters is None and depth_multiplier is not None:
+        self.depth_multiplier = depth_multiplier
+    else:
+        self.num_filters = int(kwargs.get('out_channels', num_filters))
+    self.kernel_size = kernel_size
+    self.padding = kwargs.get('padding')  # padding if padding is not None else 0in_channel
+    self.strides = kwargs['stride'] if 'stride' in kwargs else strides
+    if self.padding is not None:
+        self.auto_pad = None
+    else:
+        self.auto_pad = auto_pad
+    self.padding_mode = padding_mode
+    self.static_padding = None
+    self.dilation = dilation
+    self.transposed = transposed
+    self.groups = groups
+
+    if groups != 1 and self.num_filters % groups != 0:
+        raise ValueError('out_channels must be divisible by groups')
+
+    self.transposed = transposed
+    self.weight = None
+    self.use_bias = use_bias
+
     self.input_spec = InputSpec(ndim=self.rank + 2)
 
   def build(self, input_shape):
     input_shape = tensor_shape.TensorShape(input_shape)
     input_filters = self._get_input_channel(input_shape)
-    kernel_shape = self.kernel_size + (input_filters, self.num_filters//self.groups)
-
+    if  not self.transposed:
+        kernel_shape = self.kernel_size + (int(input_filters), int(self.num_filters)//self.groups)
+    else:
+        kernel_shape = self.kernel_size + (int(self.num_filters)//self.groups,int(input_filters))
     self.kernel = self.add_weight(
         name='kernel',
         shape=kernel_shape,
@@ -490,10 +474,9 @@ class _ConvNd(Conv):
           dtype=self.dtype)
     else:
       self.bias = None
-    channel_axis = self._get_channel_axis()
-    self.input_spec = InputSpec(ndim=self.rank + 2,
-                                axes={channel_axis: input_filters})
 
+    channel_axis = self._get_channel_axis()
+    self.input_spec = InputSpec(ndim=self.rank + 2,  axes={channel_axis: input_filters})
     self._build_conv_op_input_shape = input_shape
     self._build_input_channel = input_filters
     self._padding_op = self._get_padding_op()
@@ -501,7 +484,7 @@ class _ConvNd(Conv):
     self._convolution_op = nn_ops.Convolution(
         input_shape,
         filter_shape=self.kernel.shape,
-        dilation_rate=self.dilation_rate,
+        dilation_rate=self.dilation,
         strides=self.strides,
         padding=self._padding_op,
         data_format=self._conv_op_data_format)
@@ -526,20 +509,12 @@ class _ConvNd(Conv):
 
     # Apply causal padding to inputs for Conv1D.
     if self.padding == 'causal' and self.__class__.__name__ == 'Conv1D':
-      inputs = array_ops.pad(inputs, self._compute_causal_padding())
+      inputs = nn_ops.array_ops.pad(inputs, self._compute_causal_padding())
 
     outputs = self._convolution_op(inputs, self.kernel)
 
     if self.use_bias:
-      if self.data_format == 'channels_first':
-        if self.rank == 1:
-          # nn.bias_add does not accept a 1D input tensor.
-          bias = array_ops.reshape(self.bias, (1, self.filters, 1))
-          outputs += bias
-        else:
-          outputs = nn.bias_add(outputs, self.bias, data_format='NCHW')
-      else:
-        outputs = nn.bias_add(outputs, self.bias, data_format='NHWC')
+        outputs =Layer.bias_add(outputs, self.bias, data_format='NHWC')
 
     if self.activation is not None:
       return self.activation(outputs)
@@ -558,9 +533,21 @@ class _ConvNd(Conv):
             stride=self.strides[i],
             dilation=self.dilation_rate[i])
         new_space.append(new_dim)
-      return tensor_shape.TensorShape([input_shape[0]] + new_space + [self.filters])
+      return tensor_shape.TensorShape([input_shape[0]] + new_space +
+                                      [self.filters])
     else:
-      raise ValueError('only channel_last is supported in tensorflow')
+      space = input_shape[2:]
+      new_space = []
+      for i in range(len(space)):
+        new_dim = conv_utils.conv_output_length(
+            space[i],
+            self.kernel_size[i],
+            padding=self.padding,
+            stride=self.strides[i],
+            dilation=self.dilation_rate[i])
+        new_space.append(new_dim)
+      return tensor_shape.TensorShape([input_shape[0], self.filters] +
+                                      new_space)
 
   # def get_config(self):
   #   config = {
@@ -619,7 +606,7 @@ class _ConvNd(Conv):
 
 
 class Conv1d(tf.keras.layers.Conv1D):
-    def __init__(self, kernel_size, num_filters, strides, input_shape=None ,auto_pad=True,padding_mode=PaddingMode.replicate, activation=None, use_bias=False, dilation=1,
+    def __init__(self, kernel_size, num_filters, strides, input_shape=None ,auto_pad=True,padding_mode='replicate', activation=None, use_bias=False, dilation=1,
                  groups=1, name='', filter_rate=None,**kwargs):
         kernel_size = _single(kernel_size)
         strides = _single(strides)
