@@ -2,14 +2,14 @@
 from ..backend.load_backend import get_backend
 if get_backend()=='pytorch':
     from trident.models import pytorch_vgg
-    from trident.models import pytorch_mtcnn
+    #from trident.models import pytorch_mtcnn
     from trident.models import pytorch_resnet
     from trident.models import pytorch_densenet
     from trident.models import pytorch_efficientnet
     from trident.models import pytorch_mobilenet
     from trident.models import pytorch_gan
     from trident.models import pytorch_deeplab
-    from trident.models import pytorch_rfbnet
+    #from trident.models import pytorch_rfbnet
 
 elif get_backend()=='tensorflow':
     from trident.models import tensorflow_resnet
