@@ -26,7 +26,7 @@ from ..layers.pytorch_activations import  get_activation,Identity,Relu
 from ..layers.pytorch_normalizations import get_normalization
 from ..layers.pytorch_blocks import *
 from ..layers.pytorch_pooling import *
-from ..optims.pytorch_trainer import *
+from ..optims.pytorch_trainer import ImageClassificationModel
 from ..data.image_common import *
 from ..data.utils import download_model_from_google_drive
 
