@@ -1,11 +1,5 @@
 import os
 import sys
-import matplotlib
-import platform
-
-if platform.system() not in ['Linux', 'Darwin'] and not platform.system().startswith('CYGWIN'):
-    matplotlib.use('TKAgg')
-from IPython import display
 import inspect
 import time
 import random
@@ -25,7 +19,7 @@ from .cntk_regularizers import get_reg
 from .cntk_constraints import get_constraint
 from ..misc.visualization_utils import tile_rgb_images, loss_metric_curve
 from ..callbacks.lr_schedulers import get_lr_scheduler
-from .trainers import ModelBase,OptimizerMixin,progress_bar
+from .trainers import ModelBase,OptimizerBase,progress_bar
 from ..data.image_common import *
 __all__ = ['TrainingItem', 'Model','ImageClassificationModel','ImageDetectionModel']
 
