@@ -14,29 +14,32 @@ from ..layers.pytorch_activations import sigmoid
 from ..backend.pytorch_backend import *
 from ..backend.pytorch_ops import *
 
+
 _device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
 _session = get_session()
 
-__all__ = ['MSELoss', 'CrossEntropyLoss','NullLoss','BCELoss', 'F1ScoreLoss', 'L1Loss', 'L2Loss', 'make_onehot', 'MS_SSIM',
-           'CrossEntropyLabelSmooth', 'mixup_criterion', 'DiceLoss','IouLoss', 'focal_loss_with_logits', 'FocalLoss',
-           'SoftIoULoss', 'CenterLoss', 'TripletLoss', 'LovaszSoftmax', 'PerceptionLoss', 'EdgeLoss',
-           'TransformInvariantLoss', 'get_loss']
+__all__ = ['MSELoss', 'CrossEntropyLoss', 'NullLoss', 'BCELoss', 'F1ScoreLoss', 'L1Loss', 'SmoothL1Loss', 'L2Loss',
+           'ExponentialLoss','ItakuraSaitoLoss', 'make_onehot', 'MS_SSIMLoss', 'CrossEntropyLabelSmooth', 'mixup_criterion', 'DiceLoss',
+           'IouLoss', 'focal_loss_with_logits', 'FocalLoss', 'SoftIoULoss', 'CenterLoss', 'TripletLoss',
+           'LovaszSoftmax', 'PerceptionLoss', 'EdgeLoss', 'TransformInvariantLoss', 'get_loss']
 
 
 class CrossEntropyLoss(_Loss):
-    def __init__(self, weight=None,with_logits=True, ignore_index=-100, reduction='mean', name='CrossEntropyLoss'):
+    def __init__(self, weight=None, with_logits=True, ignore_index=-100, reduction='mean', name='CrossEntropyLoss'):
         super().__init__(reduction=reduction)
         self.name = name
-        self.weight=weight
-        self.ignore_index=ignore_index
-        self.with_logits=with_logits
+        self.weight = weight
+        self.ignore_index = ignore_index
+        self.with_logits = with_logits
 
     def forward(self, output, target):
         if not self.with_logits:
-            output=torch.softmax(output,dim=1)
-        if len(output.size())==len(target.size()):
-            target= argmax(target,1)
-        return F.cross_entropy(output, target,weight=self.weight,ignore_index=self.ignore_index,reduction=self.reduction)
+            output = torch.softmax(output, dim=1)
+        if len(output.size()) == len(target.size()):
+            target = argmax(target, 1)
+        return F.cross_entropy(output, target, weight=self.weight, ignore_index=self.ignore_index,
+                               reduction=self.reduction)
+
 
 class NullLoss(_Loss):
     def __init__(self, weight=None, with_logits=True, ignore_index=-100, reduction='mean', name='CrossEntropyLoss'):
@@ -51,17 +54,15 @@ class NullLoss(_Loss):
             output = torch.log_softmax(output, dim=1)
         if len(output.size()) == len(target.size()):
             target = argmax(target, 1)
-        return F.nll_loss(output, target, weight=self.weight, ignore_index=self.ignore_index,
-                               reduction=self.reduction)
+        return F.nll_loss(output, target, weight=self.weight, ignore_index=self.ignore_index, reduction=self.reduction)
+
+    #     if len(output.shape)==2 and len(target.shape)==1:  #         return F.cross_entropy(output, target.long(),
+    #     ignore_index=self.ignore_index,reduction=self.reduction)  #     elif len(output.shape)==2 and len(
+    #     target.shape)==2:  #         return super().forward(output, argmax(target,1))  #     elif len(
+    #     output.shape)>2 and len(target.shape)==len(output.shape)-1:  #         return super().forward(output.view(
+    #     -1,output.size(1)), target.long().view(-1))  #     elif len(output.shape)>2 and len(target.shape)==len(
+    #     output.shape):  #         return super().forward(output.view(-1,output.size(1)), argmax(target,1).view(-1))
 
-    #     if len(output.shape)==2 and len(target.shape)==1:
-    #         return F.cross_entropy(output, target.long(),ignore_index=self.ignore_index,reduction=self.reduction)
-    #     elif len(output.shape)==2 and len(target.shape)==2:
-    #         return super().forward(output, argmax(target,1))
-    #     elif len(output.shape)>2 and len(target.shape)==len(output.shape)-1:
-    #         return super().forward(output.view(-1,output.size(1)), target.long().view(-1))
-    #     elif len(output.shape)>2 and len(target.shape)==len(output.shape):
-    #         return super().forward(output.view(-1,output.size(1)), argmax(target,1).view(-1))
 
 class BCELoss(_Loss):
     def __init__(self, weight=None, reduction='mean', with_logit=False, name='BCELoss'):
@@ -72,6 +73,8 @@ class BCELoss(_Loss):
         self.with_logit = with_logit
 
     def forward(self, output, target):
+        if not self.with_logits:
+            output = torch.softmax(output, dim=1)
         target1 = to_numpy(target)
         if self.with_logit == False:
             output = sigmoid(output)
@@ -88,31 +91,99 @@ class BCELoss(_Loss):
         return F.binary_cross_entropy(output, target, weight=self.loss_weight, reduction=self.reduction)
 
 
-class L1Loss(nn.L1Loss):
+class L1Loss(_Loss):
     def __init__(self, reduction='mean', name='L1Loss'):
         super(L1Loss, self).__init__(reduction)
         self.name = name
         self.reduction = reduction
+
     def forward(self, output, target):
-        return super().forward(output, target)
+        if output.shape == target.shape:
+            return  F.l1_loss(output, target, reduction=self.reduction)
+        else:
+            raise ValueError('output and target shape should the same in L2Loss. ')
 
 
-class L2Loss(nn.MSELoss):
+class L2Loss(_Loss):
     def __init__(self, reduction='mean', name='MSELoss'):
         super(L2Loss, self).__init__(reduction)
         self.name = name
         self.reduction = reduction
+
     def forward(self, output, target):
-        return 0.5 * super(L2Loss, self)(output, target)
+        if output.shape == target.shape:
+            return 0.5 * F.mse_loss(output, target,reduction=self.reduction)
+        else:
+            raise ValueError('output and target shape should the same in L2Loss. ')
 
-
-class MSELoss(nn.MSELoss):
-    def __init__(self, reduction='mean', name='MSELoss'):
-        super(MSELoss, self).__init__(reduction)
+class SmoothL1Loss(_Loss):
+    def __init__(self, reduction='mean', name='SmoothL1Loss'):
+        super(SmoothL1Loss, self).__init__(reduction=reduction)
         self.name = name
         self.reduction = reduction
+        self.huber_delta = 0.5
+
     def forward(self, output, target):
-        return super().forward(output, target)
+        if output.shape == target.shape:
+            return F.smooth_l1_loss(output, target, reduction=self.reduction)
+        else:
+            raise ValueError('output and target shape should the same in SmoothL1Loss. ')
+
+
+class MSELoss(_Loss):
+    def __init__(self, reduction='mean', name='MSELoss'):
+        super(MSELoss, self).__init__(reduction=reduction)
+        self.name = name
+        self.reduction = reduction
+
+    def forward(self, output, target):
+        return F.mse_loss(output, target, reduction=self.reduction)
+
+
+class ExponentialLoss(_Loss):
+    def __init__(self, reduction='mean', name='ExponentialLoss'):
+        super(ExponentialLoss, self).__init__(reduction=reduction)
+        self.name = name
+        self.reduction = reduction
+
+    def forward(self, output, target):
+
+        if output.shape == target.shape:
+            error = (output - target) ** 2
+            loss = 1 - (-1 * error / np.pi).exp()
+            if self.reduction == "mean":
+                loss = loss.mean()
+            if self.reduction == "sum":
+                loss = loss.sum()
+            if self.reduction == "batchwise_mean":
+                loss = loss.mean(0).sum()
+            return loss
+        else:
+            raise ValueError('output and target shape should the same in ExponentialLoss. ')
+
+
+class ItakuraSaitoLoss(_Loss):
+    def __init__(self, reduction='mean', name='ItakuraSaitoLoss'):
+        super(ItakuraSaitoLoss, self).__init__(reduction=reduction)
+        self.name = name
+        self.reduction = reduction
+
+    def forward(self, output, target):
+        #  y_true/(y_pred+1e-12) - log(y_true/(y_pred+1e-12)) - 1;
+        if output.shape == target.shape:
+            if -1<=output.min()<0 and -1<=target.min()<0:
+                output=output+1
+                target=target+1
+            loss= (target/(output+1e-8))-((target+1e-8)/(output+1e-8)).log()-1
+            if self.reduction == "mean":
+                loss = loss.mean()
+            if self.reduction == "sum":
+                loss = loss.sum()
+            if self.reduction == "batchwise_mean":
+                loss = loss.mean(0).sum()
+            return loss
+        else:
+            raise ValueError('output and target shape should the same in ItakuraSaitoLoss. ')
 
 
 def make_onehot(labels, classes):
@@ -123,71 +194,109 @@ def make_onehot(labels, classes):
     return target
 
 
-def gaussian(window_size, sigma):
+def gaussian(window_size, sigma=1.5):
     gauss = torch.Tensor([exp(-(x - window_size // 2) ** 2 / float(2 * sigma ** 2)) for x in range(window_size)])
     return gauss / gauss.sum()
 
 
-def create_window(window_size, sigma, channel):
-    _1D_window = gaussian(window_size, sigma).unsqueeze(1)
+def create_window(window_size, channel):
+    _1D_window = gaussian(window_size, 1.5).unsqueeze(1)
     _2D_window = _1D_window.mm(_1D_window.t()).float().unsqueeze(0).unsqueeze(0)
     window = Variable(_2D_window.expand(channel, 1, window_size, window_size).contiguous())
     return window
 
 
-class MS_SSIM(_Loss):
-    def __init__(self, reduction='mean', max_val=255):
-        super(MS_SSIM, self).__init__()
+def ssim(img1, img2, window_size=11, window=None, size_average=True, full=False, val_range=None):
+    # Value range can be different from 255. Other common ranges are 1 (sigmoid) and 2 (tanh).
+    if val_range is None:
+        if torch.max(img1) > 128:
+            max_val = 255
+        else:
+            max_val = 1
+
+        if torch.min(img1) < -0.5:
+            min_val = -1
+        else:
+            min_val = 0
+        L = max_val - min_val
+    else:
+        L = val_range
+
+    padd = 0
+    (_, channel, height, width) = img1.size()
+    if window is None:
+        real_size = min(window_size, height, width)
+        window = create_window(real_size, channel=channel).to(img1.device)
+
+    mu1 = F.conv2d(img1, window, padding=padd, groups=channel)
+    mu2 = F.conv2d(img2, window, padding=padd, groups=channel)
+
+    mu1_sq = mu1.pow(2)
+    mu2_sq = mu2.pow(2)
+    mu1_mu2 = mu1 * mu2
+
+    sigma1_sq = F.conv2d(img1 * img1, window, padding=padd, groups=channel) - mu1_sq
+    sigma2_sq = F.conv2d(img2 * img2, window, padding=padd, groups=channel) - mu2_sq
+    sigma12 = F.conv2d(img1 * img2, window, padding=padd, groups=channel) - mu1_mu2
+
+    C1 = (0.01 * L) ** 2
+    C2 = (0.03 * L) ** 2
+
+    v1 = 2.0 * sigma12 + C2
+    v2 = sigma1_sq + sigma2_sq + C2
+    cs = torch.mean(v1 / v2)  # contrast sensitivity
+
+    ssim_map = ((2 * mu1_mu2 + C1) * v1) / ((mu1_sq + mu2_sq + C1) * v2)
+
+    if size_average:
+        ret = ssim_map.mean()
+    else:
+        ret = ssim_map.mean(1).mean(1).mean(1)
+
+    if full:
+        return ret, cs
+    return ret
+
+
+def msssim(img1, img2, window_size=11, size_average=True, val_range=None, normalize=False):
+    device = img1.device
+    weights = torch.FloatTensor([0.0448, 0.2856, 0.3001, 0.2363, 0.1333]).to(device)
+    levels = weights.size()[0]
+    mssim = []
+    mcs = []
+    for _ in range(levels):
+        sim, cs = ssim(img1, img2, window_size=window_size, size_average=size_average, full=True, val_range=val_range)
+        mssim.append(sim)
+        mcs.append(cs)
+
+        img1 = F.avg_pool2d(img1, (2, 2))
+        img2 = F.avg_pool2d(img2, (2, 2))
+
+    mssim = torch.stack(mssim)
+    mcs = torch.stack(mcs)
+
+    # Normalize (to avoid NaNs during training unstable models, not compliant with original definition)
+    if normalize:
+        mssim = (mssim + 1) / 2
+        mcs = (mcs + 1) / 2
+
+    pow1 = mcs ** weights
+    pow2 = mssim ** weights
+    # From Matlab implementation https://ece.uwaterloo.ca/~z70wang/research/iwssim/
+    output = torch.prod(pow1[:-1] * pow2[-1])
+    return output
+
+
+class MS_SSIMLoss(_Loss):
+    def __init__(self, reduction='mean', window_size=11, max_val=255):
+        super(MS_SSIMLoss, self).__init__()
+        self.reduction = reduction
+        self.window_size = window_size
         self.reduction = reduction
         self.channel = 3
-        self.max_val = max_val
-
-    def _ssim(self, img1, img2, size_average=True):
-        _, c, w, h = img1.size()
-        window_size = min(w, h, 11)
-        sigma = 1.5 * window_size / 11
-        window = create_window(window_size, sigma, self.channel).to(_device)
-        mu1 = F.conv2d(img1, window, padding=window_size // 2, groups=self.channel)
-        mu2 = F.conv2d(img2, window, padding=window_size // 2, groups=self.channel)
-
-        mu1_sq = mu1.pow(2)
-        mu2_sq = mu2.pow(2)
-        mu1_mu2 = mu1 * mu2
-
-        sigma1_sq = F.conv2d(img1 * img1, window, padding=window_size // 2, groups=self.channel) - mu1_sq
-        sigma2_sq = F.conv2d(img2 * img2, window, padding=window_size // 2, groups=self.channel) - mu2_sq
-        sigma12 = F.conv2d(img1 * img2, window, padding=window_size // 2, groups=self.channel) - mu1_mu2
-
-        C1 = (0.01 * self.max_val) ** 2
-        C2 = (0.03 * self.max_val) ** 2
-        V1 = 2.0 * sigma12 + C2
-        V2 = sigma1_sq + sigma2_sq + C2
-        ssim_map = ((2 * mu1_mu2 + C1) * V1) / ((mu1_sq + mu2_sq + C1) * V2)
-        mcs_map = V1 / V2
-        if self.reduction == 'mean':
-            return ssim_map.mean(), mcs_map.mean()
-        else:
-            return ssim_map.sum(), mcs_map.sum()
-
-    def ms_ssim(self, output, target, levels=5):
-
-        weight = Variable(torch.Tensor([0.0448, 0.2856, 0.3001, 0.2363, 0.1333]).cuda())
-
-        msssim = Variable(torch.Tensor(levels, ).to(_device))
-        mcs = Variable(torch.Tensor(levels, ).to(_device))
-        for i in range(levels):
-            ssim_map, mcs_map = self._ssim(output, target)
-            msssim[i] = ssim_map
-            mcs[i] = mcs_map
-            filtered_im1 = F.avg_pool2d(output, kernel_size=2, stride=2)
-            filtered_im2 = F.avg_pool2d(target, kernel_size=2, stride=2)
-            output = filtered_im1
-            target = filtered_im2
-        value = (torch.prod(mcs[0:levels - 1] ** weight[0:levels - 1]) * (msssim[levels - 1] ** weight[levels - 1]))
-        return value
 
     def forward(self, output, target):
-        return self.ms_ssim(output, target)
+        return 1-msssim(output, target, window_size=self.window_size, size_average=True, normalize=True)
 
 
 class CrossEntropyLabelSmooth(_Loss):
@@ -210,37 +319,37 @@ def mixup_criterion(y_a, y_b, lam):
     return lambda criterion, pred: lam * criterion(pred, y_a) + (1 - lam) * criterion(pred, y_b)
 
 
-
 class DiceLoss(_Loss):
-    def __init__(self,  weight=None, smooth=1., ignore_index=-100,reduction='mean'):
+    def __init__(self, weight=None, smooth=1., ignore_index=-100, reduction='mean'):
         super(DiceLoss, self).__init__(reduction=reduction)
         self.ignore_index = ignore_index
         self.smooth = smooth
-        if isinstance(weight,list):
-            weight=to_tensor(np.array(weight))
-        elif isinstance(weight,np.ndarray):
+        if isinstance(weight, list):
+            weight = to_tensor(np.array(weight))
+        elif isinstance(weight, np.ndarray):
             weight = to_tensor(weight)
-        if weight is None or isinstance(weight,torch.Tensor) :
-            self.weight=weight
+        if weight is None or isinstance(weight, torch.Tensor):
+            self.weight = weight
         else:
             raise ValueError('weight should be 1-D tensor')
 
     def forward(self, output, target):
-        if self.weight is not None and len(self.weight)!=output.size(1):
+        if self.weight is not None and len(self.weight) != output.size(1):
             raise ValueError('weight should be 1-D tensor and length equal to numbers of filters')
 
         target = make_onehot(target, classes=output.size()[1])
         probs = F.softmax(output, dim=1)
-        #probs=output#*((output>0.5).float())
-        if self.reduction=='mean':
+        # probs=output#*((output>0.5).float())
+        if self.reduction == 'mean':
             intersection = (target * probs).sum(-1).sum(-1)
-            den1=probs.sum(-1).sum(-1)
-            den2= target.sum(-1).sum(-1)
-            dice =1-((2 *intersection+ self.smooth) / (den1 + den2+self.smooth))
+            den1 = probs.sum(-1).sum(-1)
+            den2 = target.sum(-1).sum(-1)
+            dice = 1 - ((2 * intersection + self.smooth) / (den1 + den2 + self.smooth))
             if self.weight is not None:
-                dice=dice*(self.weight.unsqueeze(0))
-            dice1 = 1 - ((2 * intersection[:, 1:].sum() + self.smooth) / (den1[:, 1:].sum() + den2[:, 1:].sum() + self.smooth))
-            return dice[:, 1:].mean()#+dice1#.sum()
+                dice = dice * (self.weight.unsqueeze(0))
+            dice1 = 1 - ((2 * intersection[:, 1:].sum() + self.smooth) / (
+                        den1[:, 1:].sum() + den2[:, 1:].sum() + self.smooth))
+            return dice[:, 1:].mean()  # +dice1#.sum()
         else:
             intersection = (target * probs)[:, 1:].sum()
             den1 = probs[:, 1:].sum()
@@ -248,10 +357,12 @@ class DiceLoss(_Loss):
             dice = 1 - ((2 * intersection + self.smooth) / (den1 + den2 + self.smooth))
             return dice
 
+
 class IouLoss(_Loss):
-    def __init__(self, ignore_index=-1000,reduction='mean'):
+    def __init__(self, ignore_index=-1000, reduction='mean'):
         super(IouLoss, self).__init__(reduction=reduction)
         self.ignore_index = ignore_index
+
     def forward(self, output, target):
         output = argmax(output, 1)
         output_flat = output.reshape(-1)
@@ -260,13 +371,14 @@ class IouLoss(_Loss):
         target_flat = target_flat[target_flat != self.ignore_index]
         output_flat = output_flat[target_flat != 0]
         target_flat = target_flat[target_flat != 0]
-        intersection =(output_flat == target_flat).float().sum()
+        intersection = (output_flat == target_flat).float().sum()
         union = ((output_flat + target_flat) > 0).float().sum().clamp(min=1)
-        loss = -(intersection/union).log()
+        loss = -(intersection / union).log()
         return loss
 
+
 def focal_loss_with_logits(input: torch.Tensor, target: torch.Tensor, gamma=2.0, alpha=0.25, reduction="mean",
-        normalized=False, threshold=None, ) -> torch.Tensor:
+                           normalized=False, threshold=None, ) -> torch.Tensor:
     """Compute binary focal loss between target and output logits.
 
     See :class:`~pytorch_toolbelt.losses.FocalLoss` for details.
@@ -330,15 +442,18 @@ def focal_loss_with_logits(input: torch.Tensor, target: torch.Tensor, gamma=2.0,
 
 
 class FocalLoss(_Loss):
-    def __init__(self, alpha=0.5, gamma=2, ignore_index=None, reduction="mean", reduced=False):
+    def __init__(self, with_logits=False,alpha=0.5, gamma=2, ignore_index=None, reduction="mean"):
         super().__init__()
         self.alpha = alpha
         self.gamma = gamma
         self.ignore_index = ignore_index
         self.reduction = reduction
-        self.reduced = reduced
+        self.with_logits=with_logits
+
 
     def forward(self, output, target):
+        if not self.with_logits:
+            output=F.softmax(output,dim=1)
 
         num_classes = output.size(1)
         if target.dtype == torch.int64:
@@ -346,7 +461,7 @@ class FocalLoss(_Loss):
         loss = 0
 
         # Filter anchors with -1 label from loss computation
-        not_ignored=target
+        not_ignored = target
         if self.ignore_index is not None:
             not_ignored = target != self.ignore_index
 
@@ -357,9 +472,10 @@ class FocalLoss(_Loss):
             if self.ignore_index is not None:
                 cls_target = cls_target[not_ignored]
                 cls_input = cls_input[not_ignored]
-            loss+=focal_loss_with_logits(cls_input, cls_target, gamma=self.gamma, alpha=self.alpha, reduction=self.reduction)
+            loss += focal_loss_with_logits(cls_input, cls_target, gamma=self.gamma, alpha=self.alpha,
+                                           reduction=self.reduction)
 
-        return loss/output.size(0) if self.reduction=='mean' else loss
+        return loss / output.size(0) if self.reduction == 'mean' else loss
 
 
 class SoftIoULoss(_Loss):
@@ -618,27 +734,38 @@ class StyleLoss(nn.Module):
 
 
 class PerceptionLoss(_Loss):
-    def __init__(self, reduction="mean", reduced=False):
+    def __init__(self, vgg16, reduction="mean", reduced=False):
         super(PerceptionLoss, self).__init__()
-        self.vgg_model = torchvision.models.vgg16(pretrained=True)
+        self.vgg_model = vgg16
+        self.vgg_model.trainable = False
         self.vgg_model.eval()
-        self.vgg_layers = self.vgg_model.features
-        self.layer_name_mapping = {'3': "conv1_2", '8': "conv2_2", '15': "conv3_3", '22': "conv4_3"}
+
+        self.layer_name_mapping = {'3': "block1_conv2", '8': "block2_conv2", '15': "block3_conv3", '22': "block4_conv3"}
+        for name, module in self.vgg_model.named_modules():
+            if name in list(self.layer_name_mapping.values()):
+                module.keep_output = True
+
         self.reduction = reduction
         self.reduced = reduced
         self.styleloss = StyleLoss()
         self.to(_device)
 
     def forward(self, output, target):
-        loss = 0
-        target.detach()
-        vgg_output = self.vgg_model.features(output)
-        vgg_target = self.vgg_model.features(target)
+        target_features = OrderedDict()
+        output_features = OrderedDict()
+        _ = self.vgg_model(output)
+        for item in self.layer_name_mapping.values():
+            output_features[item] = getattr(self.vgg_model, item).output
 
-        # for name, module in self.vgg_layers._modules.items():
-        #     if name in self.layer_name_mapping:
-        #         loss =loss+ self.styleloss(module(output),module(target)).mean()
-        return self.styleloss(vgg_output, vgg_target).sum()
+        _ = self.vgg_model(target)
+        for item in self.layer_name_mapping.values():
+            target_features[item] = getattr(self.vgg_model, item).output.detach()
+
+        loss = 0
+        for i in range(len(self.layer_name_mapping)):
+            b,c,h,w=output_features.value_list[i].shape
+            loss += ((output_features.value_list[i] - target_features.value_list[i]) ** 2).mean()/(h*w)
+        return loss
 
 
 class EdgeLoss(_Loss):
@@ -736,11 +863,12 @@ class F1ScoreLoss(_Loss):
         :class:`~cntk.ops.functions.Function`
 
     """
-    def __init__(self, reduction='mean', num_class=2, ignore_index=-100,beta=1):
+
+    def __init__(self, reduction='mean', num_class=2, ignore_index=-100, beta=1):
         super(F1ScoreLoss, self).__init__(reduction=reduction)
         self.num_class = num_class
         self.ignore_index = ignore_index
-        self.beta=beta
+        self.beta = beta
 
     def forward(self, output, target):
         num_class = self.num_class
@@ -775,14 +903,16 @@ class F1ScoreLoss(_Loss):
                     precision = tp / (tp + fp + 1e-8)
                     recall = tp / (tp + fn + 1e-8)
 
-                    f1 = f1 + (1 - (1+self.beta ** 2) * (precision * recall) / (self.beta ** 2 * precision + recall + 1e-8))
+                    f1 = f1 + (1 - (1 + self.beta ** 2) * (precision * recall) / (
+                                self.beta ** 2 * precision + recall + 1e-8))
                     n += 1
             if self.reduction == 'mean':
                 f1 = f1 / n
             return f1
         else:
             raise ValueError(
-                'target.ndim:{0}  output.ndim:{1} is not valid for F1 score calculation'.format(target.ndim,   output.ndim))
+                'target.ndim:{0}  output.ndim:{1} is not valid for F1 score calculation'.format(target.ndim,
+                                                                                                output.ndim))
 
 
 def get_loss(loss_name):
@@ -797,6 +927,3 @@ def get_loss(loss_name):
         except Exception:
             loss_fn = get_class(loss_name, loss_modules)
     return loss_fn
-
-
-
