@@ -23,7 +23,7 @@ from .pytorch_regularizers import get_reg
 from .pytorch_constraints import get_constraint
 from ..misc.visualization_utils import tile_rgb_images, loss_metric_curve
 from ..callbacks.lr_schedulers import get_lr_scheduler
-from .trainers import ModelBase, OptimizerMixin, progress_bar
+from .trainers import ModelBase, OptimizerBase, progress_bar
 from ..data.image_common import *
 
 __all__ = ['TrainingItem', 'Model', 'ImageClassificationModel', 'ImageDetectionModel', 'ImageGenerationModel',
@@ -606,14 +606,13 @@ class Model(ModelBase):
                 else:
                     train_data[item] = to_tensor(train_data[item].copy())
 
-                if test_data is not None:
+                if test_data is not None and  item in test_data:
                     test_data[item] = to_tensor(test_data[item].copy())
 
                     # check target
 
             self.training_context['train_data'] = train_data
-            if test_data is not None:
-                self.training_context['test_data'] = test_data
+            self.training_context['test_data'] = test_data
 
         except:
             PrintException()
@@ -655,63 +654,78 @@ class Model(ModelBase):
 
     def log_gradient(self, grads=None, keep_gradients_history=False):
         grad_dict = {}
+        if isinstance(self._model, nn.Module):
+            for i, (k, v) in enumerate(self._model.named_parameters()):
+                grad_dict[k] = to_numpy(v.grad)
 
-        for i, (k, v) in enumerate(self._model.named_parameters()):
-            grad_dict[k] = to_numpy(v.grad)
-
-        if keep_gradients_history:
-            self.gradients_history.append(grad_dict)
+            if keep_gradients_history:
+                self.gradients_history.append(grad_dict)
 
     def log_weight(self, weghts=None, keep_weghts_history=False):
         weight_dict = {}
-        for k, v in self._model.named_parameters():
-            weight_dict[k] = to_numpy(v.data)
-        if keep_weghts_history:
-            self.weights_history.append(weight_dict)
+        if isinstance(self._model, nn.Module):
+            for k, v in self._model.named_parameters():
+                weight_dict[k] = to_numpy(v.data)
+            if keep_weghts_history:
+                self.weights_history.append(weight_dict)
 
     def save_model(self, save_path=None):
         for callback in self.training_context['callbacks']:
             callback.on_model_saving_start(self.training_context)
-        self._model.eval()
+
         if save_path is None or len(save_path) == 0:
             save_path = self.training_context.get('save_path', self.save_path)
+
+        if isinstance(self._model, nn.Module):
+            self._model.eval()
             if save_path is None or len(save_path) == 0:
-                save_path = 'Models/model_{0}_epoch{1}.pth'.format(self._model.name,
-                                                                   self.training_context['current_epoch'])
-        torch.save({'state_dict': self._model.state_dict()}, save_path + '.tar_')
-        # save safety
-        shutil.copyfile(save_path + '.tar_', save_path + '.tar')
-        self._model.train()
+                save_path = 'Models/model_{0}_epoch{1}.pth'.format(self._model.name,self.training_context['current_epoch'])
+            torch.save({'state_dict': self._model.state_dict()}, save_path + '.tar_')
+            # save safety
+            shutil.copyfile(save_path + '.tar_', save_path + '.tar')
+            self._model.train()
+        elif isinstance(self._model,torch.Tensor):
+            if save_path is None or len(save_path) == 0:
+                save_path = 'Models/model_{0}_epoch{1}.npy'.format(self.name,self.training_context['current_epoch'])
+            numpy_model=to_numpy(self._model)
+            np.save(save_path,numpy_model)
+            os.strerror('Yor model is a Tensor not a nn.Module, it has saved as numpy array(*.npy) successfully. ')
+        else:
+            raise ValueError('only Layer or nn.Module as model can export to onnx, yours model is {0}'.format(type(self._model)))
+
 
     def save_onnx(self, file_path, dynamic_axes=None):
-        import torch.onnx
-        self._model.eval()
-        dummy_input = torch.randn(1, *self._model.input_shape.tolist(),
-                                  device="cuda" if self._model.weights[0].data.is_cuda else "cpu")
-        file, ext = os.path.splitext(file_path)
-        if ext is None or ext != '.onnx':
-            file_path = os.path.join(file, '.onnx')
+        if isinstance(self._model,nn.Module):
+            import torch.onnx
+            self._model.eval()
+            dummy_input = torch.randn(1, *self._model.input_shape.tolist(),
+                                      device="cuda" if self._model.weights[0].data.is_cuda else "cpu")
+            file, ext = os.path.splitext(file_path)
+            if ext is None or ext != '.onnx':
+                file_path = os.path.join(file, '.onnx')
 
-        outputs = self._model(dummy_input)
-        if isinstance(outputs, torch.Tensor):
-            outputs = (outputs,)
-        output_names = ['output_{0}'.format(i) for i in range(len(outputs))]
-        if dynamic_axes is None:
-            dynamic_axes = {}
-            for inp in self.inputs.key_list:
-                dynamic_axes[inp] = {0: 'batch_size'}
-            for out in output_names:
-                dynamic_axes[out] = {0: 'batch_size'}
-        torch.onnx.export(self._model,  # model being run
-                          dummy_input,  # model input (or a tuple for multiple inputs)
-                          file_path,  # where to save the model (can be a file or file-like object)
-                          export_params=True,  # store the trained parameter weights inside the model file
-                          opset_version=11,  # the ONNX version to export the model to
-                          do_constant_folding=True,  # whether to execute constant folding for optimization
-                          input_names=self.inputs.key_list,  # the model's input names
-                          output_names=output_names,  # the model's output names
-                          dynamic_axes=dynamic_axes)
-        self._model.train()
+            outputs = self._model(dummy_input)
+            if isinstance(outputs, torch.Tensor):
+                outputs = (outputs,)
+            output_names = ['output_{0}'.format(i) for i in range(len(outputs))]
+            if dynamic_axes is None:
+                dynamic_axes = {}
+                for inp in self.inputs.key_list:
+                    dynamic_axes[inp] = {0: 'batch_size'}
+                for out in output_names:
+                    dynamic_axes[out] = {0: 'batch_size'}
+            torch.onnx.export(self._model,  # model being run
+                              dummy_input,  # model input (or a tuple for multiple inputs)
+                              file_path,  # where to save the model (can be a file or file-like object)
+                              export_params=True,  # store the trained parameter weights inside the model file
+                              opset_version=11,  # the ONNX version to export the model to
+                              do_constant_folding=True,  # whether to execute constant folding for optimization
+                              input_names=self.inputs.key_list,  # the model's input names
+                              output_names=output_names,  # the model's output names
+                              dynamic_axes=dynamic_axes)
+            self._model.train()
+        else:
+            raise ValueError('only Layer or nn.Module as model can export to onnx, yours model is {0}'.format(type(self._model)))
 
     def load_model(self, file_path):
         print('Loading pretrained model from {}'.format(file_path))
@@ -733,6 +747,12 @@ class Model(ModelBase):
         # self.rebinding_input_output(self._model.input_shape)
         summary(self._model, tuple(self._model.input_shape))
 
+    def predict(self,input):
+        raise NotImplementedError
+
+    def test(self, input,target):
+        raise NotImplementedError
+
     def extra_repr(self):
         return ''
 
@@ -938,20 +958,18 @@ class ImageGenerationModel(Model):
 
     def infer_single_image(self, img):
         if self._model.built:
-            self._model.to(self.device)
             self._model.eval()
-            if isinstance(img, np.ndarray) and img.ndim < 2:
-                pass
-            else:
-                img = image2array(img)
-                if img.shape[-1] == 4:
-                    img = img[:, :, :3]
+            img = image2array(img)
+            if img.shape[-1] == 4:
+                img = img[:, :, :3]
 
-                for func in self.preprocess_flow:
-                    if inspect.isfunction(func):
-                        img = func(img)
-
-            result = self._model(to_tensor(np.expand_dims(img, 0)))
+            for func in self.preprocess_flow:
+                if inspect.isfunction(func) and func is not image_backend_adaptive:
+                    img = func(img)
+            img = image_backend_adaptive(img)
+            inp = to_tensor(np.expand_dims(img, 0)).to(torch.device("cuda" if self._model.weights[0].data.is_cuda else "cpu")).to(self._model.weights[0].data.dtype)
+            result = self._model(inp)
+            result = to_numpy(result)[0]
 
             for func in self.reverse_preprocess_flow:
                 if inspect.isfunction(func):
