@@ -21,7 +21,7 @@ from tensorflow.python.keras.engine.input_spec import InputSpec
 from tensorflow.python.ops import nn_ops
 from ..backend.common import get_session,addindent,get_time_suffix,get_class,format_time,get_terminal_size,snake2camel,camel2snake
 from ..backend.tensorflow_backend import to_numpy,to_tensor,Sequential,Input
-from .trainers import OptimizerMixin
+from .trainers import OptimizerBase
 
 _session=get_session()
 _epsilon=_session.epsilon
@@ -30,34 +30,34 @@ _backend=_session.backend
 __all__ = ['Adam','RMSprop','SGD','Adagrad','Adadelta','RAdam','Lookahead','Ranger','get_optimizer']
 
 
-class Adam(tf.keras.optimizers.Adam,OptimizerMixin):
+class Adam(tf.keras.optimizers.Adam, OptimizerBase):
     # def get_value(self,x):
     #     return tf.keras.backend.get_value(x)
     # def set_value(self,x):
     #     return tf.keras.backend.set_value(x)
     pass
 
-class RMSprop(tf.keras.optimizers.RMSprop,OptimizerMixin):
+class RMSprop(tf.keras.optimizers.RMSprop, OptimizerBase):
     def get_value(self,x):
         return tf.keras.backend.get_value(x)
     def set_value(self,x):
         return tf.keras.backend.set_value(x)
     pass
 
-class SGD(tf.keras.optimizers.SGD,OptimizerMixin):
+class SGD(tf.keras.optimizers.SGD, OptimizerBase):
     def get_value(self,x):
         return tf.keras.backend.get_value(x)
     def set_value(self,x):
         return tf.keras.backend.set_value(x)
     pass
 
-class Adagrad(tf.keras.optimizers.Adagrad,OptimizerMixin):
+class Adagrad(tf.keras.optimizers.Adagrad, OptimizerBase):
     def get_value(self,x):
         return tf.keras.backend.get_value(x)
     def set_value(self,x):
         return tf.keras.backend.set_value(x)
     pass
-class Adadelta(tf.keras.optimizers.Adadelta,OptimizerMixin):
+class Adadelta(tf.keras.optimizers.Adadelta, OptimizerBase):
     def get_value(self,x):
         return tf.keras.backend.get_value(x)
     def set_value(self,x):
@@ -67,7 +67,7 @@ class Adadelta(tf.keras.optimizers.Adadelta,OptimizerMixin):
 
 
 
-class RAdam(tf.keras.optimizers.Optimizer,OptimizerMixin):
+class RAdam(tf.keras.optimizers.Optimizer, OptimizerBase):
     """Variant of the Adam optimizer whose adaptive learning rate is rectified
     so as to have a consistent variance.
 
@@ -357,7 +357,7 @@ class RAdam(tf.keras.optimizers.Optimizer,OptimizerMixin):
         return config
 
 
-class Lookahead(tf.keras.optimizers.Optimizer,OptimizerMixin):
+class Lookahead(tf.keras.optimizers.Optimizer, OptimizerBase):
     """This class allows to extend optims with the lookahead mechanism.
 
     The mechanism is proposed by Michael R. Zhang et.al in the paper
@@ -510,7 +510,7 @@ class Lookahead(tf.keras.optimizers.Optimizer,OptimizerMixin):
         return cls(optimizer, **config)
 
 
-class Ranger(tf.keras.optimizers.Optimizer,OptimizerMixin):
+class Ranger(tf.keras.optimizers.Optimizer, OptimizerBase):
     """This class allows to extend optims with the lookahead mechanism.
 
     The mechanism is proposed by Michael R. Zhang et.al in the paper
