@@ -1,14 +1,5 @@
 import os
 import sys
-import matplotlib
-import platform
-
-if platform.system() not in ['Linux', 'Darwin'] and not platform.system().startswith('CYGWIN'):
-    matplotlib.use('TKAgg')
-from ..misc.ipython_utils import is_in_ipython
-if is_in_ipython():
-    from IPython import display
-
 import inspect
 import time
 import uuid
@@ -32,7 +23,7 @@ from .tensorflow_metrics import get_metric
 from ..misc.visualization_utils import tile_rgb_images, loss_metric_curve
 from ..callbacks import *
 from ..callbacks.lr_schedulers import get_lr_scheduler
-from .trainers import ModelBase, OptimizerMixin, progress_bar
+from .trainers import ModelBase, OptimizerBase, progress_bar
 from ..backend.common import *
 from ..backend.tensorflow_backend import Input, Sequential, to_numpy, to_tensor, print_summary
 from ..data.image_common import *
