@@ -29,7 +29,7 @@ from ..callbacks.visualization_callbacks import *
 from ..misc.ipython_utils import *
 from ..misc.visualization_utils import tile_rgb_images, loss_metric_curve
 
-__all__ = ['progress_bar', 'OptimizerMixin', 'ModelBase', 'TrainingPlan']
+__all__ = ['progress_bar', 'OptimizerBase', 'ModelBase', 'TrainingPlan']
 
 _session = get_session()
 _backend = _session.backend
@@ -52,23 +52,7 @@ last_time = time.time()
 begin_time = last_time
 
 
-class OptimizerMixin(object):
-    # def __init__(self, parameters,lr=0.01,**kwargs):
-    #     if _backend == 'pytorch':
-    #         kwargs['lr']=lr
-    #         self.base_lr=lr
-    #         super(parameters,kwargs)
-    #     elif _backend == 'tensorflow':
-    #         super().set_value(self.lr, lr)
-    #         kwargs['learning_rate '] = lr
-    #         super(kwargs)
-    #         self.parameters=parameters
-    #     elif _backend == 'cntk':
-    #         kwargs['lr'] = lr
-    #         self.base_lr = lr
-    #         super(parameters, **kwargs)
-    #     self.optimizer=super()
-
+class OptimizerBase(object):
     def adjust_learning_rate(self, new_lr, verbose=True):
         if _backend in ['cntk', 'pytorch']:
             old_lr = self.param_groups[0]['lr']
@@ -226,9 +210,11 @@ class ModelBase(object):
         self.training_context = {'losses': OrderedDict(),  # loss_wrapper
                                  'metrics': OrderedDict(),  # loss_wrapper
                                  'grads_state': OrderedDict(),  # loss_wrapper
-                                 'tmp_losses': OrderedDict(),  # loss_wrapper
+                                 'tmp_losses': [],  # loss_wrapper
                                  'tmp_metrics': OrderedDict(),  # loss_wrapper
-                                 'grads': None, 'optimizer': None,  # optimizer
+                                 'out_sample_metrics': OrderedDict(),
+                                 'grads': None,
+                                 'optimizer': None,  # optimizer
                                  'stop_training': False,  # stop training
                                  'total_epoch': -1,  # current_epoch
                                  'total_batch': -1,  # current_batch
@@ -434,11 +420,12 @@ class ModelBase(object):
         return self
 
     def reset_training_context(self):
-        self.training_context = {'losses': OrderedDict(),  # loss_wrapper
-                                 'metrics': OrderedDict(),  # loss_wrapper
-                                 'grads_state': OrderedDict(),  # loss_wrapper
-                                 'tmp_losses': OrderedDict(),  # loss_wrapper
-                                 'tmp_metrics': OrderedDict(),  # loss_wrapper
+        self.training_context = {'losses': OrderedDict(),
+                                 'metrics': OrderedDict(),
+                                 'grads_state': OrderedDict(),
+                                 'tmp_losses': [],
+                                 'tmp_metrics': OrderedDict(),
+                                 'out_sample_metrics': OrderedDict(),
                                  'grads': None, 'optimizer': None,  # optimizer
                                  'stop_training': False,  # stop training
                                  'total_epoch': -1,  # current_epoch
@@ -735,23 +722,36 @@ class ModelBase(object):
                     if isinstance(output, (list, tuple)):
                         for i in range(len(output)):
                             test_data[self.outputs.key_list[i]] = output[i]
+                    elif 'tensor' in output.__class__.__name__.lower():
+                        test_data[self.outputs.key_list[0]] = output
                     else:
                         test_data[self.outputs.key_list[0]] = output
 
+
+
                 # ON_EVALUATION_START
                 self.do_on_metrics_evaluation_start()
                 for callback in self.training_context['callbacks']:
                     callback.on_metrics_evaluation_start(self.training_context)
 
+
                 for k, v in self._metrics.items():
                     if k not in self.training_context['metrics']:
                         self.training_context['tmp_metrics'][k] = []
                         self.training_context['metrics'][k] = []
 
-                    this_metric = try_map_args_and_call(v, test_data if test_data is not None and len(test_data) > 0 else train_data, self.training_context['data_feed'])
-
+                    this_metric = try_map_args_and_call(v, train_data, self.training_context['data_feed'])
                     self.training_context['tmp_metrics'][k].append(to_numpy(this_metric).mean())
 
+                    if test_data is not None and len(test_data) > 0 :
+                        if k not in self.training_context['out_sample_metrics']:
+                            self.training_context['out_sample_metrics'][k] = []
+
+                        this_out_metric = try_map_args_and_call(v, test_data , self.training_context['data_feed'])
+                        self.training_context['out_sample_metrics'][k].append(to_numpy(this_out_metric).mean())
+
+
+
                 # ON_EVALUATION_END
                 self.do_on_metrics_evaluation_end()
                 for callback in self.training_context['callbacks']:
@@ -774,6 +774,7 @@ class ModelBase(object):
                         callback.on_progress_start(self.training_context)
 
                     self.print_batch_progress(self.training_context['print_batch_progress_frequency'])
+
                     self.training_context['print_batch_progress_frequency'] = 1
                     self.do_on_progress_end()
                     for callback in self.training_context['callbacks']:
@@ -781,6 +782,10 @@ class ModelBase(object):
                 else:
                     self.training_context['print_batch_progress_frequency'] += 1
 
+                if test_data is not None and len(test_data) > 0:
+                    print('out-of-sample evaluation: ',','.join(['{0}: {1:<8.3%}'.format(k, v[-1]) for k, v in self.training_context['out_sample_metrics'].items()]))
+
+
                 # ON_BATCH_END
                 self.do_on_batch_end()
                 for callback in self.training_context['callbacks']:
@@ -820,6 +825,23 @@ class ModelBase(object):
     def summary(self):
         raise NotImplementedError
 
+    def predict(self,input):
+        if isinstance(input,(torch.Tensor,np.ndarray)):
+            if isinstance(input,np.ndarray):
+                input=to_tensor(input)
+            if len(input.shape)==len(self.inputs.value_list[0])+1:
+                return self._model(input.unsqueeze(0))
+            elif len(input.shape)==len(self.inputs.value_list[0]):
+                return self._model(input)
+            else:
+                return None
+        else:
+            raise NotImplementedError
+
+
+    def test(self, input,target):
+        raise NotImplementedError
+
 
 class TrainingPlan(object):
     def __init__(self):
@@ -833,6 +855,9 @@ class TrainingPlan(object):
         self.print_progress_frequency = 10
         self.print_progress_unit = 'batch'
         self.print_progress_on_epoch_end = False
+        self.out_sample_evaluation_frequency = 100
+        self.out_sample_evaluation_unit = 'batch'
+        self.out_sample_evaluation_on_epoch_end = True
         self.save_model_frequency = -1
         self.save_model_unit = 'batch'
         self.execution_id = None
@@ -972,6 +997,15 @@ class TrainingPlan(object):
         self.minibatch_size = minibatch_size
         return self
 
+    def out_sample_evaluation_scheduling(self, frequency: int, unit='batch', on_epoch_end=True):
+        self.out_sample_evaluation_on_epoch_end = on_epoch_end
+        self.out_sample_evaluation_frequency = frequency
+        if unit not in ['batch', 'epoch']:
+            raise ValueError('unit should be batch or epoch')
+        else:
+            self.out_sample_evaluation_unit = unit
+
+        return self
     def print_progress_scheduling(self, frequency: int, unit='batch', on_epoch_end=True, show_loss_metric_curve=True):
         self.print_progress_on_epoch_end = on_epoch_end
         self.print_progress_frequency = frequency
@@ -1029,43 +1063,48 @@ class TrainingPlan(object):
         self.callbacks.append(plot)
         return self
 
-    def generate_datafeed(self):
+    def generate_datafeed(self,data_loader):
+        if data_loader.signature is None:
+            _ = data_loader.next()
         for trainingitem in self.training_items.value_list:
-            data_feed=OrderedDict()
-            outputs=trainingitem.outputs
-            targets=trainingitem.targets
-            if len(self._dataloaders)==1:
-                data_loader = self._dataloaders.value_list[0]
-                if data_loader.signature is None:
-                    _=data_loader.next()
-                if len(trainingitem.model.signature)+len(targets)==len(data_loader.signature):
-                    if len(outputs)==1 and len(trainingitem.model.signature)==1  :
-
-                        if  1<=len(data_loader.signature)<=2:
-                                data_feed[trainingitem.model.signature.key_list[0]]=data_loader.signature.key_list[0]
-                                for loss in trainingitem._losses.value_list:
-                                    args=get_signature(loss)
-                                    if hasattr(loss,'signature'):
-                                        args=loss.signature
-                                    if len(args)==2:
-                                        if args.key_list[0] not in data_feed or (args.key_list[0] in data_feed and data_feed[args.key_list[0]] is None):
-                                            data_feed[args.key_list[0]]=outputs.key_list[0]
-                                        if args.key_list[1] not in data_feed or (args.key_list[1] in data_feed and data_feed[args.key_list[1]] is None):
-                                            if args.key_list[1] in data_loader.signature.key_list:
-                                                data_feed[args.key_list[1]] = args.key_list[1]
-                                            else:
-                                                data_feed[args.key_list[1]] =data_loader.signature.key_list[-1]   #-1 is for handel autoencoder scenario
-                                    else:
-                                        raise ValueError('loss shoud only 2 argments when one-input-one-output model with 2 dataset items in data loaders')
-
-                                trainingitem.training_context['data_feed']=data_feed
-                                print('data_feed for {0} :{1}'.format(trainingitem.name,data_feed))
-
-
-                else:
-                    raise RuntimeError('the number of models input plus the numbers of  targets should equal to the numbers of dataset items')
+            data_feed =None
+            if 'data_feed' in trainingitem.training_context and  all([  True for key in data_loader.signature.key_list if key in trainingitem.training_context['data_feed'].value_list]):
+                data_feed = trainingitem.training_context['data_feed']
             else:
-                raise  RuntimeError('Multiple data loader data_feed auto-generation is not support Now.')
+                if 'data_feed' in trainingitem.training_context:
+                    data_feed = trainingitem.training_context['data_feed']
+                else:
+                    data_feed=OrderedDict()
+                outputs=trainingitem.outputs
+                targets=trainingitem.targets
+                if len(self._dataloaders)==1:
+                    if len(trainingitem.model.signature)+len(targets)==len(data_loader.signature) or len(data_loader.signature)==1:
+                        if len(outputs)==1 and len(trainingitem.model.signature)==1  :
+                            if  1<=len(data_loader.signature)<=2:
+                                    data_feed[trainingitem.model.signature.key_list[0]]=data_loader.signature.key_list[0]
+                                    for loss in trainingitem._losses.value_list:
+                                        args=get_signature(loss)
+                                        if hasattr(loss,'signature'):
+                                            args=loss.signature
+                                        if len(args)==2:
+                                            if args.key_list[0] not in data_feed or (args.key_list[0] in data_feed and data_feed[args.key_list[0]] is None):
+                                                data_feed[args.key_list[0]]=outputs.key_list[0]
+                                            if args.key_list[1] not in data_feed or (args.key_list[1] in data_feed and data_feed[args.key_list[1]] is None):
+                                                if args.key_list[1] in data_loader.signature.key_list:
+                                                    data_feed[args.key_list[1]] = args.key_list[1]
+                                                else:
+                                                    data_feed[args.key_list[1]] =data_loader.signature.key_list[-1]   #-1 is for handel autoencoder scenario
+                                        else:
+                                            raise ValueError('loss shoud only 2 argments when one-input-one-output model with 2 dataset items in data loaders')
+
+                                    trainingitem.training_context['data_feed']=data_feed
+                                    print('data_feed for {0} :{1}'.format(trainingitem.name,data_feed))
+                        #elif len(outputs)==1
+
+                    else:
+                        raise RuntimeError('the number of models input plus the numbers of  targets should equal to the numbers of dataset items')
+                else:
+                    raise  RuntimeError('Multiple data loader data_feed auto-generation is not support Now.')
 
 
 
@@ -1078,20 +1117,25 @@ class TrainingPlan(object):
             self.execution_id = str(uuid.uuid4())[:8].__str__().replace('-', '') if self.execution_id is None else self.execution_id
             # update callback
             if not is_resume:
-                self.generate_datafeed()
                 for item in self.training_items.values():
                     for callback in self.callbacks:
                         if callback not in item.callbacks:
+                            #private callback
                             if callback.is_shared == False:
                                 item.with_callbacks(copy.deepcopy(callback))
                             else:
+                                #shared callback
                                 item.with_callbacks(callback)
+                #shared callbacks will access training plan dict instead of training_context
                 for callback in self.callbacks:
                     if callback.is_shared == True:
                         callback.on_training_start(self.__dict__)
 
             data_loader = self._dataloaders.value_list[0]
             data_loader.minibatch_size = self.minibatch_size
+            #generate data feed
+            self.generate_datafeed(data_loader)
+
             if not is_resume:
                 #print(self.__repr__)
                 if collect_data_inteval == 1 and len(data_loader.batch_sampler) * self.num_epochs > 1000:
@@ -1109,8 +1153,18 @@ class TrainingPlan(object):
                             name = data_loader.signature.key_list[i]
                             iter_data[name] = return_data[i]
 
+                        #check weather need out-of-sample evaluation
+                        need_out_sample_evaluation=False
+                        if self.out_sample_evaluation_on_epoch_end==True and mbs==len(data_loader.batch_sampler)-1:
+                            need_out_sample_evaluation=True
+                        if self.out_sample_evaluation_unit=='batch' and mbs>0 and mbs%self.out_sample_evaluation_frequency==0:
+                            need_out_sample_evaluation = True
+                        elif self.out_sample_evaluation_unit=='epoch' and mbs==len(data_loader.batch_sampler)-1 and epoch%self.out_sample_evaluation_frequency==0:
+                            need_out_sample_evaluation = True
+
+
                         iter_testdata = None
-                        if isinstance(data_loader, DataProviderV2):
+                        if isinstance(data_loader, DataProviderV2) and data_loader.testdata is not None and need_out_sample_evaluation:
                             return_test = data_loader.next_test()
                             if return_test is not None:
                                 iter_testdata = OrderedDict()
